@@ -1,20 +1,24 @@
-/*
-   Copyright (c) 2001, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2001, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
 /*
   UNION  of select's
@@ -22,21 +26,55 @@
 */
 
 
-#include "mysql_priv.h"
+#include "sql_priv.h"
+#include "unireg.h"
+#include "sql_union.h"
 #include "sql_select.h"
 #include "sql_cursor.h"
+#include "sql_base.h"                           // fill_record
+#include "filesort.h"                           // filesort_free_buffers
+#include "sql_tmp_table.h"                      // tmp tables
+#include "sql_optimizer.h"                      // JOIN
+#include "opt_explain_format.h"
 
 bool mysql_union(THD *thd, LEX *lex, select_result *result,
                  SELECT_LEX_UNIT *unit, ulong setup_tables_done_option)
 {
-  DBUG_ENTER("mysql_union");
   bool res;
-  if (!(res= unit->prepare(thd, result, SELECT_NO_UNLOCK |
-                           setup_tables_done_option)))
-    res= unit->exec();
-  if (res || !thd->cursor || !thd->cursor->is_open())
-    res|= unit->cleanup();
+  DBUG_ENTER("mysql_union");
+
+  res= unit->prepare(thd, result,
+		     SELECT_NO_UNLOCK | setup_tables_done_option);
+  if (res)
+    goto err;
+
+
+  /*
+    Tables are not locked at this point, it means that we have delayed
+    this step until after prepare stage (i.e. this moment). This allows to
+    do better partition pruning and avoid locking unused partitions.
+    As a consequence, in such a case, prepare stage can rely only on
+    metadata about tables used and not data from them.
+    We need to lock tables now in order to proceed with the remaning
+    stages of query optimization and execution.
+  */
+  DBUG_ASSERT(! thd->lex->is_query_tables_locked());
+  if (lock_tables(thd, lex->query_tables, lex->table_count, 0))
+    goto err;
+
+  /*
+    Tables must be locked before storing the query in the query cache.
+    Transactional engines must been signalled that the statement started,
+    which external_lock signals.
+  */
+  query_cache_store_query(thd, thd->lex->query_tables);
+
+  res= unit->optimize() || unit->exec();
+  res|= unit->cleanup();
   DBUG_RETURN(res);
+err:
+  (void) unit->cleanup();
+  DBUG_RETURN(true);
 }
 
 
@@ -59,7 +97,7 @@ bool select_union::send_data(List<Item> &values)
     unit->offset_limit_cnt--;
     return 0;
   }
-  fill_record(thd, table->field, values, 1);
+  fill_record(thd, table->field, values, 1, NULL);
   if (thd->is_error())
     return 1;
 
@@ -67,7 +105,8 @@ bool select_union::send_data(List<Item> &values)
   {
     /* create_myisam_from_heap will generate error if needed */
     if (table->file->is_fatal_error(error, HA_CHECK_DUP) &&
-        create_myisam_from_heap(thd, table, &tmp_table_param, error, 1))
+        create_myisam_from_heap(thd, table, tmp_table_param.start_recinfo, 
+                                &tmp_table_param.recinfo, error, TRUE, NULL))
       return 1;
   }
   return 0;
@@ -102,6 +141,8 @@ bool select_union::flush()
       is_union_distinct  if set, the temporary table will eliminate
                          duplicates on insert
       options            create options
+      table_alias        name of the temporary table
+      bit_fields_as_long convert bit fields to ulonglong
 
   DESCRIPTION
     Create a temporary table that is used to store the result of a UNION,
@@ -115,36 +156,60 @@ bool select_union::flush()
 bool
 select_union::create_result_table(THD *thd_arg, List<Item> *column_types,
                                   bool is_union_distinct, ulonglong options,
-                                  const char *alias)
+                                  const char *table_alias,
+                                  bool bit_fields_as_long, bool create_table)
 {
   DBUG_ASSERT(table == 0);
   tmp_table_param.init();
   tmp_table_param.field_count= column_types->elements;
+  tmp_table_param.skip_create_table= !create_table;
+  tmp_table_param.bit_fields_as_long= bit_fields_as_long;
 
   if (! (table= create_tmp_table(thd_arg, &tmp_table_param, *column_types,
                                  (ORDER*) 0, is_union_distinct, 1,
-                                 options, HA_POS_ERROR, (char*) alias)))
+                                 options, HA_POS_ERROR, (char*) table_alias)))
     return TRUE;
-  table->file->extra(HA_EXTRA_WRITE_CACHE);
-  table->file->extra(HA_EXTRA_IGNORE_DUP_KEY);
+  if (create_table)
+  {
+    table->file->extra(HA_EXTRA_WRITE_CACHE);
+    table->file->extra(HA_EXTRA_IGNORE_DUP_KEY);
+  }
   return FALSE;
 }
 
 
-/*
-  initialization procedures before fake_select_lex preparation()
+/**
+  Reset and empty the temporary table that stores the materialized query result.
 
-  SYNOPSIS
-    st_select_lex_unit::init_prepare_fake_select_lex()
-    thd		- thread handler
-
-  RETURN
-    options of SELECT
+  @note The cleanup performed here is exactly the same as for the two temp
+  tables of JOIN - exec_tmp_table_[1 | 2].
 */
 
-void
-st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg) 
+void select_union::cleanup()
 {
+  table->file->extra(HA_EXTRA_RESET_STATE);
+  table->file->ha_delete_all_rows();
+  free_io_cache(table);
+  filesort_free_buffers(table,0);
+}
+
+
+/**
+  Initialization procedures before fake_select_lex preparation()
+
+  @param thd		 Thread handler
+  @param no_const_tables Skip reading const tables. TRUE for EXPLAIN.
+
+  @returns
+    TRUE  OOM
+    FALSE Ok
+*/
+
+bool
+st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg,
+                                                 bool no_const_tables)
+{
+  DBUG_ENTER("st_select_lex_unit::init_prepare_fake_select_lex");
   thd_arg->lex->current_select= fake_select_lex;
   fake_select_lex->table_list.link_in_list(&result_table_list,
                                            &result_table_list.next_local);
@@ -165,6 +230,42 @@ st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg)
     (*order->item)->walk(&Item::change_context_processor, 0,
                          (uchar*) &fake_select_lex->context);
   }
+  if (!fake_select_lex->join)
+  {
+    /*
+      allocate JOIN for fake select only once (prevent
+      mysql_select automatic allocation)
+      TODO: The above is nonsense. mysql_select() will not allocate the
+      join if one already exists. There must be some other reason why we
+      don't let it allocate the join. Perhaps this is because we need
+      some special parameter values passed to join constructor?
+    */
+    if (!(fake_select_lex->join=
+        new JOIN(thd, item_list, fake_select_lex->options, result)))
+    {
+      fake_select_lex->table_list.empty();
+      DBUG_RETURN(true);
+    }
+    fake_select_lex->join->init(thd, item_list, fake_select_lex->options,
+                                result);
+    fake_select_lex->join->no_const_tables= no_const_tables;
+
+    /*
+      Fake st_select_lex should have item list for correct ref_array
+      allocation.
+    */
+    fake_select_lex->item_list= item_list;
+
+    /*
+      We need to add up n_sum_items in order to make the correct
+      allocation in setup_ref_array().
+      Don't add more sum_items if we have already done JOIN::prepare
+      for this (with a different join object)
+    */
+    if (fake_select_lex->ref_pointer_array.is_null())
+      fake_select_lex->n_child_sum_items+= global_parameters->n_sum_items;
+  }
+  DBUG_RETURN(false);
 }
 
 
@@ -177,7 +278,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
   bool is_union_select;
   DBUG_ENTER("st_select_lex_unit::prepare");
 
-  describe= test(additional_options & SELECT_DESCRIBE);
+  describe= MY_TEST(additional_options & SELECT_DESCRIBE);
 
   /*
     result object should be reassigned even if preparing already done for
@@ -192,17 +293,18 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
       /* fast reinit for EXPLAIN */
       for (sl= first_sl; sl; sl= sl->next_select())
       {
-	sl->join->result= result;
-	select_limit_cnt= HA_POS_ERROR;
-	offset_limit_cnt= 0;
-	if (!sl->join->procedure &&
-	    result->prepare(sl->join->fields_list, this))
-	{
-	  DBUG_RETURN(TRUE);
-	}
-	sl->join->select_options|= SELECT_DESCRIBE;
-	sl->join->reinit();
+        sl->join->result= result;
+        select_limit_cnt= HA_POS_ERROR;
+        offset_limit_cnt= 0;
+        if (result->prepare(sl->join->fields_list, this))
+        {
+          DBUG_RETURN(TRUE);
+        }
+        sl->join->select_options|= SELECT_DESCRIBE;
+        sl->join->reset();
       }
+      if (fake_select_lex->join)
+        fake_select_lex->join->result= result;
     }
     DBUG_RETURN(FALSE);
   }
@@ -232,7 +334,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
     bool can_skip_order_by;
     sl->options|=  SELECT_NO_UNLOCK;
     JOIN *join= new JOIN(thd_arg, sl->item_list, 
-			 sl->options | thd_arg->options | additional_options,
+			 sl->options | thd_arg->variables.option_bits | additional_options,
 			 tmp_result);
     /*
       setup_tables_done_option should be set only for very first SELECT,
@@ -248,8 +350,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
 
     can_skip_order_by= is_union_select && !(sl->braces && sl->explicit_limit);
 
-    saved_error= join->prepare(&sl->ref_pointer_array,
-                               sl->table_list.first,
+    saved_error= join->prepare(sl->table_list.first,
                                sl->with_wild,
                                sl->where,
                                (can_skip_order_by ? 0 :
@@ -259,12 +360,9 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
                                NULL : sl->order_list.first,
                                sl->group_list.first,
                                sl->having,
-                               (is_union_select ? NULL :
-                                thd_arg->lex->proc_list.first),
                                sl, this);
     /* There are no * in the statement anymore (for PS) */
     sl->with_wild= 0;
-    last_procedure= join->procedure;
 
     if (saved_error || (saved_error= thd_arg->is_fatal_error))
       goto err;
@@ -281,6 +379,19 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
       Item *item_tmp;
       while ((item_tmp= it++))
       {
+        /*
+          If the outer query has a GROUP BY clause, an outer reference to this
+          query block may have been wrapped in a Item_outer_ref, which has not
+          been fixed yet. An Item_type_holder must be created based on a fixed
+          Item, so use the inner Item instead.
+        */
+        DBUG_ASSERT(item_tmp->fixed ||
+                    (item_tmp->type() == Item::REF_ITEM &&
+                     ((Item_ref *)(item_tmp))->ref_type() ==
+                     Item_ref::OUTER_REF));
+        if (!item_tmp->fixed)
+          item_tmp= item_tmp->real_item();
+
 	/* Error's in 'new' will be detected after loop */
 	types.push_back(new Item_type_holder(thd_arg, item_tmp));
       }
@@ -312,6 +423,10 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
     /*
       Check that it was possible to aggregate
       all collations together for UNION.
+      We need this in case of UNION DISTINCT, to filter
+      out duplicates using the proper collation.
+
+      TODO: consider removing this test in case of UNION ALL.
     */
     List_iterator_fast<Item> tp(types);
     Item *type;
@@ -356,7 +471,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
     }
 
 
-    create_options= (first_sl->options | thd_arg->options |
+    create_options= (first_sl->options | thd_arg->variables.option_bits |
                      TMP_TABLE_ALL_COLUMNS);
     /*
       Force the temporary table to be a MyISAM table if we're going to use
@@ -367,10 +482,10 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
     if (global_parameters->ftfunc_list->elements)
       create_options= create_options | TMP_TABLE_FORCE_MYISAM;
 
-    if (union_result->create_result_table(thd, &types, test(union_distinct),
-                                          create_options, ""))
+    if (union_result->create_result_table(thd, &types, MY_TEST(union_distinct),
+                                          create_options, "", FALSE, TRUE))
       goto err;
-    bzero((char*) &result_table_list, sizeof(result_table_list));
+    memset(&result_table_list, 0, sizeof(result_table_list));
     result_table_list.db= (char*) "";
     result_table_list.table_name= result_table_list.alias= (char*) "union";
     result_table_list.table= table= union_result->table;
@@ -378,42 +493,30 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
     thd_arg->lex->current_select= lex_select_save;
     if (!item_list.elements)
     {
-      Query_arena *arena, backup_arena;
+      {
+        Prepared_stmt_arena_holder ps_arena_holder(thd);
 
-      arena= thd->activate_stmt_arena_if_needed(&backup_arena);
-      
-      saved_error= table->fill_item_list(&item_list);
+        saved_error= table->fill_item_list(&item_list);
 
-      if (arena)
-        thd->restore_active_arena(arena, &backup_arena);
-
-      if (saved_error)
-        goto err;
+        if (saved_error)
+          goto err;
+      }
 
       if (thd->stmt_arena->is_stmt_prepare())
       {
         /* Validate the global parameters of this union */
+        init_prepare_fake_select_lex(thd, false);
 
-	init_prepare_fake_select_lex(thd);
-        /* Should be done only once (the only item_list per statement) */
-        DBUG_ASSERT(fake_select_lex->join == 0);
-	if (!(fake_select_lex->join= new JOIN(thd, item_list, thd->options,
-					      result)))
-	{
-	  fake_select_lex->table_list.empty();
-	  DBUG_RETURN(TRUE);
-	}
-	fake_select_lex->item_list= item_list;
-
-	thd_arg->lex->current_select= fake_select_lex;
 	saved_error= fake_select_lex->join->
-	  prepare(&fake_select_lex->ref_pointer_array,
-		  fake_select_lex->table_list.first,
-		  0, 0,
-		  fake_select_lex->order_list.elements,
-		  fake_select_lex->order_list.first,
-		  NULL, NULL, NULL,
-		  fake_select_lex, this);
+	  prepare(fake_select_lex->table_list.first, // tables_init
+                  0,                                 // wild_num
+                  0,                                 // conds_init
+                  global_parameters->order_list.elements, // og_num
+                  global_parameters->order_list.first,    // order
+                  NULL,                                // group_init
+                  NULL,                                // having_init
+                  fake_select_lex,                     // select_lex_arg
+                  this);                               // unit_arg
 	fake_select_lex->table_list.empty();
       }
     }
@@ -433,23 +536,168 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
 
 err:
   thd_arg->lex->current_select= lex_select_save;
+  (void) cleanup();
   DBUG_RETURN(TRUE);
 }
 
 
+/**
+  Run optimization phase.
+
+  @return FALSE unit successfully passed optimization phase.
+  @return TRUE an error occur.
+*/
+
+bool st_select_lex_unit::optimize()
+{
+  DBUG_ENTER("st_select_lex_unit::optimize");
+
+  if (optimized && item && item->assigned() && !uncacheable && !describe)
+    DBUG_RETURN(FALSE);
+
+  for (SELECT_LEX *sl= first_select(); sl; sl= sl->next_select())
+  {
+    DBUG_ASSERT(sl->join);
+    if (optimized)
+    {
+      saved_error= false;
+      sl->join->reset();
+    }
+    else
+    {
+      SELECT_LEX *lex_select_save= thd->lex->current_select;
+      thd->lex->current_select= sl;
+      set_limit(sl);
+      if ((sl == global_parameters && is_union()) || describe)
+      {
+        offset_limit_cnt= 0;
+        /*
+          We can't use LIMIT at this stage if we are using ORDER BY for the
+          whole UNION.
+        */
+        if (sl->order_list.first || describe)
+          select_limit_cnt= HA_POS_ERROR;
+      }
+
+      /*
+        When using braces, SQL_CALC_FOUND_ROWS affects the whole query:
+        we don't calculate found_rows() per union part.
+        Otherwise, SQL_CALC_FOUND_ROWS should be done on all sub parts.
+      */
+      sl->join->select_options= 
+        (select_limit_cnt == HA_POS_ERROR || sl->braces) ?
+        sl->options & ~OPTION_FOUND_ROWS : sl->options | found_rows_for_union;
+
+      saved_error= sl->join->optimize();
+      /*
+        Accumulate estimated number of rows.
+        1. Implicitly grouped query has one row (with HAVING it has zero or one
+           rows).
+        2. If GROUP BY clause is optimized away because it was a constant then
+           query produces at most one row.
+      */
+      result->estimated_rowcount+=
+        (sl->with_sum_func && sl->group_list.elements == 0) ||
+        sl->join->group_optimized_away ?
+          1 : sl->join->best_rowcount;
+
+      thd->lex->current_select= lex_select_save;
+    }
+    if (saved_error)
+      break;
+  }
+  if (!saved_error)
+    optimized= 1;
+
+  DBUG_RETURN(saved_error);
+}
+
+
+/**
+  Explain UNION.
+*/
+
+bool st_select_lex_unit::explain()
+{
+  SELECT_LEX *lex_select_save= thd->lex->current_select;
+  Explain_format *fmt= thd->lex->explain_format;
+  DBUG_ENTER("st_select_lex_unit::explain");
+  JOIN *join;
+  bool ret= false;
+
+  DBUG_ASSERT((is_union() || fake_select_lex) && describe && optimized);
+  executed= true;
+
+  if (fmt->begin_context(CTX_UNION))
+    DBUG_RETURN(true);
+
+  for (SELECT_LEX *sl= first_select(); sl; sl= sl->next_select())
+  {
+    if (fmt->begin_context(CTX_QUERY_SPEC))
+      DBUG_RETURN(true);
+    DBUG_ASSERT(sl->join);
+    if (sl->join->explain() || thd->is_error())
+      DBUG_RETURN(true);
+    if (fmt->end_context(CTX_QUERY_SPEC))
+      DBUG_RETURN(true);
+  }
+
+  if (init_prepare_fake_select_lex(thd, true))
+    DBUG_RETURN(true);
+
+  if (thd->is_fatal_error)
+    DBUG_RETURN(true);
+  join= fake_select_lex->join;
+
+  /*
+    In EXPLAIN command, constant subqueries that do not use any
+    tables are executed two times:
+     - 1st time is a real evaluation to get the subquery value
+     - 2nd time is to produce EXPLAIN output rows.
+    1st execution sets certain members (e.g. select_result) to perform
+    subquery execution rather than EXPLAIN line production. In order 
+    to reset them back, we re-do all of the actions (yes it is ugly).
+  */
+  if (!join->optimized || !join->tables)
+  {
+    saved_error= mysql_select(thd,
+                          &result_table_list,
+                          0, item_list, NULL,
+                          &global_parameters->order_list,
+                          NULL, NULL,
+                          fake_select_lex->options | SELECT_NO_UNLOCK,
+                          result, this, fake_select_lex);
+  }
+  else
+    ret= join->explain();
+
+  thd->lex->current_select= lex_select_save;
+
+  if (saved_error || ret || thd->is_error())
+    DBUG_RETURN(true);
+  fmt->end_context(CTX_UNION);
+
+  DBUG_RETURN(false);
+}
+
+
+/**
+  Execute UNION.
+*/
+
 bool st_select_lex_unit::exec()
 {
   SELECT_LEX *lex_select_save= thd->lex->current_select;
-  SELECT_LEX *select_cursor=first_select();
   ulonglong add_rows=0;
   ha_rows examined_rows= 0;
   DBUG_ENTER("st_select_lex_unit::exec");
+  DBUG_ASSERT((is_union() || fake_select_lex) && !describe && optimized);
 
-  if (executed && !uncacheable && !describe)
-    DBUG_RETURN(FALSE);
-  executed= 1;
+  if (executed && !uncacheable)
+    DBUG_RETURN(false);
+  executed= true;
   
-  if (uncacheable || !item || !item->assigned() || describe)
+  if (uncacheable || !item || !item->assigned())
   {
     if (item)
       item->reset_value_registration();
@@ -467,211 +715,171 @@ bool st_select_lex_unit::exec()
         DBUG_ASSERT(0);
       }
     }
-    for (SELECT_LEX *sl= select_cursor; sl; sl= sl->next_select())
+
+    for (SELECT_LEX *sl= first_select(); sl; sl= sl->next_select())
     {
       ha_rows records_at_start= 0;
+      DBUG_ASSERT(sl->join);
       thd->lex->current_select= sl;
 
-      if (optimized)
-	saved_error= sl->join->reinit();
-      else
+      set_limit(sl);
+      if (sl == global_parameters || describe)
       {
-        set_limit(sl);
-	if (sl == global_parameters || describe)
-	{
-	  offset_limit_cnt= 0;
-	  /*
-	    We can't use LIMIT at this stage if we are using ORDER BY for the
-	    whole query
-	  */
-	  if (sl->order_list.first || describe)
-	    select_limit_cnt= HA_POS_ERROR;
-        }
-
+        offset_limit_cnt= 0;
         /*
-          When using braces, SQL_CALC_FOUND_ROWS affects the whole query:
-          we don't calculate found_rows() per union part.
-          Otherwise, SQL_CALC_FOUND_ROWS should be done on all sub parts.
+          We can't use LIMIT at this stage if we are using ORDER BY for the
+          whole query
         */
-        sl->join->select_options= 
-          (select_limit_cnt == HA_POS_ERROR || sl->braces) ?
-          sl->options & ~OPTION_FOUND_ROWS : sl->options | found_rows_for_union;
-	saved_error= sl->join->optimize();
+        if (sl->order_list.first || describe)
+          select_limit_cnt= HA_POS_ERROR;
       }
       if (!saved_error)
       {
-	records_at_start= table->file->stats.records;
-	sl->join->exec();
+        records_at_start= table->file->stats.records;
+        sl->join->exec();
         if (sl == union_distinct)
-	{
-	  if (table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL))
-	    DBUG_RETURN(TRUE);
-	  table->no_keyread=1;
-	}
-	saved_error= sl->join->error;
-	offset_limit_cnt= (ha_rows)(sl->offset_limit ?
+        {
+          if (table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL))
+            DBUG_RETURN(true);
+          table->no_keyread=1;
+        }
+        saved_error= sl->join->error;
+        offset_limit_cnt= (ha_rows)(sl->offset_limit ?
                                     sl->offset_limit->val_uint() :
                                     0);
-	if (!saved_error)
-	{
-	  examined_rows+= thd->examined_row_count;
-	  if (union_result->flush())
-	  {
-	    thd->lex->current_select= lex_select_save;
-	    DBUG_RETURN(1);
-	  }
-	}
+        if (!saved_error)
+        {
+          /*
+            Save the current examined row count locally and clear the global
+            counter, so that we can accumulate the number of evaluated rows for
+            the current query block.
+          */
+	  examined_rows+= thd->get_examined_row_count();
+          thd->set_examined_row_count(0);
+
+          if (union_result->flush())
+          {
+            thd->lex->current_select= lex_select_save;
+            DBUG_RETURN(true);
+          }
+        }
       }
       if (saved_error)
       {
-	thd->lex->current_select= lex_select_save;
-	DBUG_RETURN(saved_error);
+        thd->lex->current_select= lex_select_save;
+        DBUG_RETURN(saved_error);
       }
       /* Needed for the following test and for records_at_start in next loop */
       int error= table->file->info(HA_STATUS_VARIABLE);
       if(error)
       {
         table->file->print_error(error, MYF(0));
-        DBUG_RETURN(1);
+        DBUG_RETURN(true);
       }
       if (found_rows_for_union && !sl->braces && 
           select_limit_cnt != HA_POS_ERROR)
       {
-	/*
-	  This is a union without braces. Remember the number of rows that
-	  could also have been part of the result set.
-	  We get this from the difference of between total number of possible
-	  rows and actual rows added to the temporary table.
-	*/
-	add_rows+= (ulonglong) (thd->limit_found_rows - (ulonglong)
-			      ((table->file->stats.records -  records_at_start)));
+        /*
+          This is a union without braces. Remember the number of rows that
+          could also have been part of the result set.
+          We get this from the difference of between total number of possible
+          rows and actual rows added to the temporary table.
+        */
+        add_rows+= (ulonglong) (thd->limit_found_rows -
+                   (ulonglong)(table->file->stats.records - records_at_start));
       }
     }
   }
-  optimized= 1;
 
-  /* Send result to 'result' */
-  saved_error= TRUE;
+  if (!saved_error && !thd->is_fatal_error)
   {
+    /* Send result to 'result' */
+    saved_error= true;
     List<Item_func_match> empty_list;
     empty_list.empty();
 
-    if (!thd->is_fatal_error)				// Check if EOM
+    set_limit(global_parameters);
+    if (init_prepare_fake_select_lex(thd, true))
+      DBUG_RETURN(true);
+    JOIN *join= fake_select_lex->join;
+    if (!join->optimized)
     {
-      set_limit(global_parameters);
-      init_prepare_fake_select_lex(thd);
-      JOIN *join= fake_select_lex->join;
-      if (!join)
-      {
-	/*
-	  allocate JOIN for fake select only once (prevent
-	  mysql_select automatic allocation)
-          TODO: The above is nonsense. mysql_select() will not allocate the
-          join if one already exists. There must be some other reason why we
-          don't let it allocate the join. Perhaps this is because we need
-          some special parameter values passed to join constructor?
-	*/
-	if (!(fake_select_lex->join= new JOIN(thd, item_list,
-					      fake_select_lex->options, result)))
-	{
-	  fake_select_lex->table_list.empty();
-	  DBUG_RETURN(TRUE);
-	}
-        fake_select_lex->join->no_const_tables= TRUE;
-
-	/*
-	  Fake st_select_lex should have item list for correctref_array
-	  allocation.
-	*/
-	fake_select_lex->item_list= item_list;
-        saved_error= mysql_select(thd, &fake_select_lex->ref_pointer_array,
-                              &result_table_list,
-                              0, item_list, NULL,
-                              global_parameters->order_list.elements,
-                              global_parameters->order_list.first,
-                              NULL, NULL, NULL,
-                              fake_select_lex->options | SELECT_NO_UNLOCK,
-                              result, this, fake_select_lex);
-      }
-      else
-      {
-        if (describe)
-        {
-          /*
-            In EXPLAIN command, constant subqueries that do not use any
-            tables are executed two times:
-             - 1st time is a real evaluation to get the subquery value
-             - 2nd time is to produce EXPLAIN output rows.
-            1st execution sets certain members (e.g. select_result) to perform
-            subquery execution rather than EXPLAIN line production. In order 
-            to reset them back, we re-do all of the actions (yes it is ugly):
-          */
-	  join->init(thd, item_list, fake_select_lex->options, result);
-          saved_error= mysql_select(thd, &fake_select_lex->ref_pointer_array,
-                                &result_table_list,
-                                0, item_list, NULL,
-                                global_parameters->order_list.elements,
-                                global_parameters->order_list.first,
-                                NULL, NULL, NULL,
-                                fake_select_lex->options | SELECT_NO_UNLOCK,
-                                result, this, fake_select_lex);
-        }
-        else
-        {
-          join->examined_rows= 0;
-          saved_error= join->reinit();
-          join->exec();
-        }
-      }
-
-      fake_select_lex->table_list.empty();
-      if (!saved_error)
-      {
-	thd->limit_found_rows = (ulonglong)table->file->stats.records + add_rows;
-        thd->examined_row_count+= examined_rows;
-      }
-      /*
-	Mark for slow query log if any of the union parts didn't use
-	indexes efficiently
-      */
+      saved_error=
+        mysql_select(thd,
+                     &result_table_list,      // tables
+                     0,                       // wild_num
+                     item_list,               // fields
+                     NULL,                    // conds
+                     &global_parameters->order_list,    // order
+                     NULL,                    // group
+                     NULL,                    // having
+                     fake_select_lex->options | SELECT_NO_UNLOCK,
+                     result,                  // result
+                     this,                    // unit
+                     fake_select_lex);        // select_lex
     }
+    else
+    {
+      join->examined_rows= 0;
+      saved_error= false;
+      join->reset();
+      join->exec();
+    }
+
+    fake_select_lex->table_list.empty();
+  }
+  if (!saved_error && !thd->is_fatal_error)
+  {
+
+    thd->limit_found_rows = (ulonglong)table->file->stats.records + add_rows;
+    thd->inc_examined_row_count(examined_rows);
   }
   thd->lex->current_select= lex_select_save;
   DBUG_RETURN(saved_error);
 }
 
 
+/**
+  Cleanup this query expression object after preparation or one round
+  of execution. After the cleanup, the object can be reused for a
+  new round of execution, but a new optimization will be needed before
+  the execution.
+
+  @return false if previous execution was successful, and true otherwise
+*/
+
 bool st_select_lex_unit::cleanup()
 {
-  int error= 0;
+  bool error= false;
   DBUG_ENTER("st_select_lex_unit::cleanup");
 
   if (cleaned)
   {
     DBUG_RETURN(FALSE);
   }
-  cleaned= 1;
-
-  if (union_result)
-  {
-    delete union_result;
-    union_result=0; // Safety
-    if (table)
-      free_tmp_table(thd, table);
-    table= 0; // Safety
-  }
+  cleaned= true;
 
   for (SELECT_LEX *sl= first_select(); sl; sl= sl->next_select())
     error|= sl->cleanup();
 
+  cleanup_level();
+
+  DBUG_RETURN(error);
+}
+
+
+/**
+  Cleanup only this select_lex_unit after preparation or one round of
+  execution.
+
+  @return false if previous execution was successful, and true otherwise
+*/
+bool st_select_lex_unit::cleanup_level()
+{
+  bool error= false;
+
   if (fake_select_lex)
   {
-    JOIN *join;
-    if ((join= fake_select_lex->join))
-    {
-      join->tables_list= 0;
-      join->tables= 0;
-    }
     error|= fake_select_lex->cleanup();
     /*
       There are two cases when we should clean order items:
@@ -693,7 +901,18 @@ bool st_select_lex_unit::cleanup()
     }
   }
 
-  DBUG_RETURN(error);
+  if (union_result)
+  {
+    delete union_result;
+    union_result=0; // Safety
+    if (table)
+      free_tmp_table(thd, table);
+    table= 0; // Safety
+  }
+
+  explain_marker= CTX_NONE;
+
+  return error;
 }
 
 
@@ -732,8 +951,8 @@ void st_select_lex_unit::reinit_exec_mechanism()
     TRUE  - error
 */
 
-bool st_select_lex_unit::change_result(select_subselect *new_result,
-                                       select_subselect *old_result)
+bool st_select_lex_unit::change_result(select_result_interceptor *new_result,
+                                       select_result_interceptor *old_result)
 {
   bool res= FALSE;
   for (SELECT_LEX *sl= first_select(); sl; sl= sl->next_select())
@@ -767,17 +986,6 @@ bool st_select_lex_unit::change_result(select_subselect *new_result,
 
 List<Item> *st_select_lex_unit::get_unit_column_types()
 {
-  SELECT_LEX *sl= first_select();
-  bool is_procedure= test(sl->join->procedure);
-
-  if (is_procedure)
-  {
-    /* Types for "SELECT * FROM t1 procedure analyse()"
-       are generated during execute */
-    return &sl->join->procedure_fields_list;
-  }
-
-
   if (is_union())
   {
     DBUG_ASSERT(prepared);
@@ -785,14 +993,66 @@ List<Item> *st_select_lex_unit::get_unit_column_types()
     return &types;
   }
 
-  return &sl->item_list;
+  return &first_select()->item_list;
 }
 
+
+/**
+  Get field list for this query expression.
+
+  For a UNION of query blocks, return the field list generated
+  during prepare.
+  For a single query block, return the field list after all possible
+  intermediate query processing steps are completed.
+
+  @returns List containing fields of the query expression.
+*/
+
+List<Item> *st_select_lex_unit::get_field_list()
+{
+  if (is_union())
+  {
+    DBUG_ASSERT(prepared);
+    /* Types are generated during prepare */
+    return &types;
+  }
+
+  return first_select()->join->fields;
+}
+
+
+/**
+  Cleanup after preparation or one round of execution.
+
+  @return false if previous execution was successful, and true otherwise
+*/
+
 bool st_select_lex::cleanup()
 {
   bool error= FALSE;
   DBUG_ENTER("st_select_lex::cleanup()");
 
+  error= cleanup_level();
+  for (SELECT_LEX_UNIT *lex_unit= first_inner_unit(); lex_unit;
+       lex_unit= lex_unit->next_unit())
+  {
+    error|= lex_unit->cleanup();
+  }
+
+  DBUG_RETURN(error);
+}
+
+
+/**
+  Cleanup only this select_lex after preparation or one round of
+  execution.
+
+  @return false if previous execution was successful, and true otherwise
+*/
+bool st_select_lex::cleanup_level()
+{
+  bool error= FALSE;
+
   if (join)
   {
     DBUG_ASSERT((st_select_lex*)join->select_lex == this);
@@ -800,14 +1060,12 @@ bool st_select_lex::cleanup()
     delete join;
     join= 0;
   }
-  for (SELECT_LEX_UNIT *lex_unit= first_inner_unit(); lex_unit ;
-       lex_unit= lex_unit->next_unit())
-  {
-    error= (bool) ((uint) error | (uint) lex_unit->cleanup());
-  }
+
+  cur_pos_in_all_fields= ALL_FIELDS_UNDEF_POS;
   non_agg_fields.empty();
   inner_refs_list.empty();
-  DBUG_RETURN(error);
+
+  return error;
 }
 
 
