@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -44,6 +51,9 @@ Transporter::Transporter(TransporterRegistry &t_reg,
     isServer(lNodeId==serverNodeId),
     m_packer(_signalId, _checksum), m_max_send_buffer(max_send_buffer),
     m_overload_limit(0xFFFFFFFF), m_slowdown_limit(0xFFFFFFFF),
+    m_bytes_sent(0), m_bytes_received(0),
+    m_connect_count(0),
+    m_overload_count(0), m_slowdown_count(0),
     isMgmConnection(_isMgmConnection),
     m_connected(false),
     m_type(_type),
@@ -77,16 +87,13 @@ Transporter::Transporter(TransporterRegistry &t_reg,
   m_timeOutMillis = 3000;
 
   m_connect_address.s_addr= 0;
-  if(s_port<0)
-    s_port= -s_port; // was dynamic
 
   if (isServer)
     m_socket_client= 0;
   else
   {
-    m_socket_client= new SocketClient(remoteHostName, s_port,
-				      new SocketAuthSimple("ndbd",
-							   "ndbd passwd"));
+    m_socket_client= new SocketClient(new SocketAuthSimple("ndbd",
+                                                           "ndbd passwd"));
 
     m_socket_client->set_connect_timeout(m_timeOutMillis);
   }
@@ -148,8 +155,10 @@ Transporter::connect_server(NDB_SOCKET_TYPE sockfd,
     DBUG_RETURN(false);
   }
 
-  m_connected  = true;
+  m_connect_count++;
+  resetCounters();
 
+  m_connected  = true;
   DBUG_RETURN(true);
 }
 
@@ -162,9 +171,20 @@ Transporter::connect_client() {
   if(m_connected)
     DBUG_RETURN(true);
 
+  int port = m_s_port;
+  if (port<0)
+  {
+    // The port number is stored as negative to indicate it's a port number
+    // which the server side setup dynamically and thus was communicated
+    // to the client via the ndb_mgmd.
+    // Reverse the negative port number to get the connectable port
+    port= -port;
+  }
+
   if(isMgmConnection)
   {
-    sockfd= m_transporter_registry.connect_ndb_mgmd(m_socket_client);
+    sockfd= m_transporter_registry.connect_ndb_mgmd(remoteHostName,
+                                                    port);
   }
   else
   {
@@ -179,7 +199,9 @@ Transporter::connect_client() {
       if (m_socket_client->bind(localHostName, 0) != 0)
         DBUG_RETURN(false);
     }
-    sockfd= m_socket_client->connect();
+
+    sockfd= m_socket_client->connect(remoteHostName,
+                                     port);
   }
 
   DBUG_RETURN(connect_client(sockfd));
@@ -274,8 +296,10 @@ Transporter::connect_client(NDB_SOCKET_TYPE sockfd) {
   if (!connect_client_impl(sockfd))
     DBUG_RETURN(false);
 
-  m_connected = true;
+  m_connect_count++;
+  resetCounters();
 
+  m_connected = true;
   DBUG_RETURN(true);
 }
 
@@ -286,7 +310,14 @@ Transporter::doDisconnect() {
     return;
 
   m_connected = false;
-
   disconnectImpl();
 }
 
+void
+Transporter::resetCounters()
+{
+  m_bytes_sent = 0;
+  m_bytes_received = 0;
+  m_overload_count = 0;
+  m_slowdown_count = 0;
+};
