@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -20,6 +27,9 @@
 #include "Dbdih.hpp"
 #include <ndb_limits.h>
 
+#define JAM_FILE_ID 355
+
+
 #define DEBUG(x) { ndbout << "DIH::" << x << endl; }
 
 void Dbdih::initData() 
@@ -41,17 +51,18 @@ void Dbdih::initData()
   for(i = 0; i<MAX_NDB_NODES; i++){
     new (&nodeRecord[i]) NodeRecord();
   }
-  
-  c_takeOverPool.setSize(MAX_NDB_NODES);
+  Uint32 max_takeover_threads = MAX(MAX_NDB_NODES,
+                                    ZMAX_TAKE_OVER_THREADS);
+  c_takeOverPool.setSize(max_takeover_threads);
   {
     Ptr<TakeOverRecord> ptr;
-    while (c_activeTakeOverList.seize(ptr))
+    while (c_masterActiveTakeOverList.seizeFirst(ptr))
     {
       new (ptr.p) TakeOverRecord;
     }
-    while (c_activeTakeOverList.first(ptr))
+    while (c_masterActiveTakeOverList.first(ptr))
     {
-      releaseTakeOver(ptr);
+      releaseTakeOver(ptr, true);
     }
   }
   
@@ -70,17 +81,24 @@ void Dbdih::initData()
   cntrlblockref    = RNIL;
   c_set_initial_start_flag = FALSE;
   c_sr_wait_to = false;
-
+  c_2pass_inr = false;
+  c_handled_master_take_over_copy_gci = 0;
+  
   c_lcpTabDefWritesControl.init(MAX_CONCURRENT_LCP_TAB_DEF_FLUSHES);
+
+  c_shutdownReqNodes.clear();
 }//Dbdih::initData()
 
-void Dbdih::initRecords() 
+void Dbdih::initRecords()
 {
   // Records with dynamic sizes
-  apiConnectRecord = (ApiConnectRecord*)
-    allocRecord("ApiConnectRecord", 
-                sizeof(ApiConnectRecord),
-                capiConnectFileSize);
+  for (Uint32 i = 0; i < c_diverify_queue_cnt; i++)
+  {
+    c_diverify_queue[i].apiConnectRecord = (ApiConnectRecord*)
+      allocRecord("ApiConnectRecord",
+                  sizeof(ApiConnectRecord),
+                  capiConnectFileSize);
+  }
 
   connectRecord = (ConnectRecord*)allocRecord("ConnectRecord",
                                               sizeof(ConnectRecord), 
@@ -98,9 +116,7 @@ void Dbdih::initRecords()
                                   sizeof(PageRecord), 
                                   cpageFileSize);
 
-  replicaRecord = (ReplicaRecord*)allocRecord("ReplicaRecord",
-                                              sizeof(ReplicaRecord), 
-                                              creplicaFileSize);
+  c_replicaRecordPool.setSize(creplicaFileSize);
 
   tabRecord = (TabRecord*)allocRecord("TabRecord",
                                               sizeof(TabRecord), 
@@ -127,12 +143,47 @@ void Dbdih::initRecords()
 
 Dbdih::Dbdih(Block_context& ctx):
   SimulatedBlock(DBDIH, ctx),
+  c_queued_lcp_frag_rep(c_replicaRecordPool),
   c_activeTakeOverList(c_takeOverPool),
+  c_queued_for_start_takeover_list(c_takeOverPool),
+  c_queued_for_commit_takeover_list(c_takeOverPool),
+  c_active_copy_threads_list(c_takeOverPool),
+  c_completed_copy_threads_list(c_takeOverPool),
+  c_masterActiveTakeOverList(c_takeOverPool),
   c_waitGCPProxyList(waitGCPProxyPool),
-  c_waitGCPMasterList(waitGCPMasterPool)
+  c_waitGCPMasterList(waitGCPMasterPool),
+  c_waitEpochMasterList(waitGCPMasterPool)
 {
   BLOCK_CONSTRUCTOR(Dbdih);
 
+  c_mainTakeOverPtr.i = RNIL;
+  c_mainTakeOverPtr.p = 0;
+  c_activeThreadTakeOverPtr.i = RNIL;
+  c_activeThreadTakeOverPtr.p = 0;
+
+  /* Node Recovery Status Module signals */
+  addRecSignal(GSN_ALLOC_NODEID_REP, &Dbdih::execALLOC_NODEID_REP);
+  addRecSignal(GSN_INCL_NODE_HB_PROTOCOL_REP,
+               &Dbdih::execINCL_NODE_HB_PROTOCOL_REP);
+  addRecSignal(GSN_NDBCNTR_START_WAIT_REP,
+               &Dbdih::execNDBCNTR_START_WAIT_REP);
+  addRecSignal(GSN_NDBCNTR_STARTED_REP,
+               &Dbdih::execNDBCNTR_STARTED_REP);
+  addRecSignal(GSN_SUMA_HANDOVER_COMPLETE_REP,
+               &Dbdih::execSUMA_HANDOVER_COMPLETE_REP);
+  addRecSignal(GSN_END_TOREP, &Dbdih::execEND_TOREP);
+  addRecSignal(GSN_LOCAL_RECOVERY_COMP_REP,
+               &Dbdih::execLOCAL_RECOVERY_COMP_REP);
+  addRecSignal(GSN_DBINFO_SCANREQ, &Dbdih::execDBINFO_SCANREQ);
+  /* End Node Recovery Status Module signals */
+
+  /* LCP Pause module */
+  addRecSignal(GSN_PAUSE_LCP_REQ, &Dbdih::execPAUSE_LCP_REQ);
+  addRecSignal(GSN_PAUSE_LCP_CONF, &Dbdih::execPAUSE_LCP_CONF);
+  addRecSignal(GSN_FLUSH_LCP_REP_REQ, &Dbdih::execFLUSH_LCP_REP_REQ);
+  addRecSignal(GSN_FLUSH_LCP_REP_CONF, &Dbdih::execFLUSH_LCP_REP_CONF);
+  /* End LCP Pause module */
+
   addRecSignal(GSN_DUMP_STATE_ORD, &Dbdih::execDUMP_STATE_ORD);
   addRecSignal(GSN_NDB_TAMPER, &Dbdih::execNDB_TAMPER, true);
   addRecSignal(GSN_DEBUG_SIG, &Dbdih::execDEBUG_SIG);
@@ -140,6 +191,8 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_MASTER_GCPREF, &Dbdih::execMASTER_GCPREF);
   addRecSignal(GSN_MASTER_GCPCONF, &Dbdih::execMASTER_GCPCONF);
   addRecSignal(GSN_EMPTY_LCP_CONF, &Dbdih::execEMPTY_LCP_CONF);
+  addRecSignal(GSN_EMPTY_LCP_REP, &Dbdih::execEMPTY_LCP_REP);
+
   addRecSignal(GSN_MASTER_LCPREQ, &Dbdih::execMASTER_LCPREQ);
   addRecSignal(GSN_MASTER_LCPREF, &Dbdih::execMASTER_LCPREF);
   addRecSignal(GSN_MASTER_LCPCONF, &Dbdih::execMASTER_LCPCONF);
@@ -168,8 +221,10 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_START_COPYREQ, &Dbdih::execSTART_COPYREQ);
   addRecSignal(GSN_START_COPYCONF, &Dbdih::execSTART_COPYCONF);
   addRecSignal(GSN_START_COPYREF, &Dbdih::execSTART_COPYREF);
-  addRecSignal(GSN_CREATE_FRAGREQ, &Dbdih::execCREATE_FRAGREQ);
-  addRecSignal(GSN_CREATE_FRAGCONF, &Dbdih::execCREATE_FRAGCONF);
+  addRecSignal(GSN_UPDATE_FRAG_STATEREQ,
+                 &Dbdih::execUPDATE_FRAG_STATEREQ);
+  addRecSignal(GSN_UPDATE_FRAG_STATECONF,
+                 &Dbdih::execUPDATE_FRAG_STATECONF);
   addRecSignal(GSN_DIVERIFYREQ, &Dbdih::execDIVERIFYREQ);
   addRecSignal(GSN_GCP_SAVEREQ, &Dbdih::execGCP_SAVEREQ);
   addRecSignal(GSN_GCP_SAVEREF, &Dbdih::execGCP_SAVEREF);
@@ -186,6 +241,7 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_COPY_GCICONF, &Dbdih::execCOPY_GCICONF);
   addRecSignal(GSN_COPY_TABREQ, &Dbdih::execCOPY_TABREQ);
   addRecSignal(GSN_COPY_TABCONF, &Dbdih::execCOPY_TABCONF);
+  addRecSignal(GSN_CHECK_LCP_IDLE_ORD, &Dbdih::execCHECK_LCP_IDLE_ORD);
   addRecSignal(GSN_TCGETOPSIZECONF, &Dbdih::execTCGETOPSIZECONF);
   addRecSignal(GSN_TC_CLOPSIZECONF, &Dbdih::execTC_CLOPSIZECONF);
 
@@ -203,11 +259,11 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_COPY_FRAGREF, &Dbdih::execCOPY_FRAGREF);
   addRecSignal(GSN_DIADDTABREQ, &Dbdih::execDIADDTABREQ);
   addRecSignal(GSN_DIGETNODESREQ, &Dbdih::execDIGETNODESREQ);
-  addRecSignal(GSN_DIRELEASEREQ, &Dbdih::execDIRELEASEREQ);
-  addRecSignal(GSN_DISEIZEREQ, &Dbdih::execDISEIZEREQ);
   addRecSignal(GSN_STTOR, &Dbdih::execSTTOR);
-  addRecSignal(GSN_DI_FCOUNTREQ, &Dbdih::execDI_FCOUNTREQ);
-  addRecSignal(GSN_DIGETPRIMREQ, &Dbdih::execDIGETPRIMREQ);
+  addRecSignal(GSN_DIH_SCAN_TAB_REQ, &Dbdih::execDIH_SCAN_TAB_REQ);
+  addRecSignal(GSN_DIH_SCAN_GET_NODES_REQ, &Dbdih::execDIH_SCAN_GET_NODES_REQ);
+  addRecSignal(GSN_DIH_SCAN_TAB_COMPLETE_REP,
+               &Dbdih::execDIH_SCAN_TAB_COMPLETE_REP);
   addRecSignal(GSN_GCP_TCFINISHED, &Dbdih::execGCP_TCFINISHED);
   addRecSignal(GSN_READ_NODESCONF, &Dbdih::execREAD_NODESCONF);
   addRecSignal(GSN_NDB_STTOR, &Dbdih::execNDB_STTOR);
@@ -219,6 +275,8 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_START_FRAGCONF, &Dbdih::execSTART_FRAGCONF);
   addRecSignal(GSN_ADD_FRAGCONF, &Dbdih::execADD_FRAGCONF);
   addRecSignal(GSN_ADD_FRAGREF, &Dbdih::execADD_FRAGREF);
+  addRecSignal(GSN_DROP_FRAG_REF, &Dbdih::execDROP_FRAG_REF);
+  addRecSignal(GSN_DROP_FRAG_CONF, &Dbdih::execDROP_FRAG_CONF);
   addRecSignal(GSN_FSOPENCONF, &Dbdih::execFSOPENCONF);
   addRecSignal(GSN_FSOPENREF, &Dbdih::execFSOPENREF, true);
   addRecSignal(GSN_FSCLOSECONF, &Dbdih::execFSCLOSECONF);
@@ -237,6 +295,9 @@ Dbdih::Dbdih(Block_context& ctx):
 
   addRecSignal(GSN_CHECKNODEGROUPSREQ, &Dbdih::execCHECKNODEGROUPSREQ);
 
+  addRecSignal(GSN_CHECK_NODE_RESTARTREQ,
+               &Dbdih::execCHECK_NODE_RESTARTREQ);
+
   addRecSignal(GSN_BLOCK_COMMIT_ORD,
 	       &Dbdih::execBLOCK_COMMIT_ORD);
   addRecSignal(GSN_UNBLOCK_COMMIT_ORD,
@@ -264,8 +325,6 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_WAIT_GCP_CONF, &Dbdih::execWAIT_GCP_CONF);
 
   addRecSignal(GSN_PREP_DROP_TAB_REQ, &Dbdih::execPREP_DROP_TAB_REQ);
-  addRecSignal(GSN_WAIT_DROP_TAB_REF, &Dbdih::execWAIT_DROP_TAB_REF);
-  addRecSignal(GSN_WAIT_DROP_TAB_CONF, &Dbdih::execWAIT_DROP_TAB_CONF);
   addRecSignal(GSN_DROP_TAB_REQ, &Dbdih::execDROP_TAB_REQ);
 
   addRecSignal(GSN_ALTER_TAB_REQ, &Dbdih::execALTER_TAB_REQ);
@@ -288,25 +347,53 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_UPGRADE_PROTOCOL_ORD,
 	       &Dbdih::execUPGRADE_PROTOCOL_ORD);
 
-  apiConnectRecord = 0;
+  addRecSignal(GSN_CREATE_NODEGROUP_IMPL_REQ,
+               &Dbdih::execCREATE_NODEGROUP_IMPL_REQ);
+
+  addRecSignal(GSN_DROP_NODEGROUP_IMPL_REQ,
+               &Dbdih::execDROP_NODEGROUP_IMPL_REQ);
+
+
+  addRecSignal(GSN_DIH_GET_TABINFO_REQ,
+               &Dbdih::execDIH_GET_TABINFO_REQ);
+#if 0
+  addRecSignal(GSN_DIH_GET_TABINFO_REF,
+               &Dbdih::execDIH_GET_TABINFO_REF);
+  addRecSignal(GSN_DIH_GET_TABINFO_CONF,
+               &Dbdih::execDIH_GET_TABINFO_CONF);
+#endif
+
   connectRecord = 0;
   fileRecord = 0;
   fragmentstore = 0;
   pageRecord = 0;
-  replicaRecord = 0;
   tabRecord = 0;
   createReplicaRecord = 0;
   nodeGroupRecord = 0;
   nodeRecord = 0;
   c_nextNodeGroup = 0;
+  c_fragments_per_node_ = 0;
+  bzero(c_node_groups, sizeof(c_node_groups));
+  if (globalData.ndbMtTcThreads == 0)
+  {
+    c_diverify_queue_cnt = 1;
+  }
+  else
+  {
+    c_diverify_queue_cnt = globalData.ndbMtTcThreads;
+  }
 }//Dbdih::Dbdih()
 
-Dbdih::~Dbdih() 
+Dbdih::~Dbdih()
 {
-  deallocRecord((void **)&apiConnectRecord, "ApiConnectRecord", 
-                sizeof(ApiConnectRecord),
-                capiConnectFileSize);
-  
+  for (Uint32 i = 0; i<c_diverify_queue_cnt; i++)
+  {
+    deallocRecord((void **)&c_diverify_queue[i].apiConnectRecord,
+                  "ApiConnectRecord",
+                  sizeof(ApiConnectRecord),
+                  capiConnectFileSize);
+  }
+
   deallocRecord((void **)&connectRecord, "ConnectRecord",
                 sizeof(ConnectRecord), 
                 cconnectFileSize);
@@ -323,10 +410,6 @@ Dbdih::~Dbdih()
                 sizeof(PageRecord), 
                 cpageFileSize);
   
-  deallocRecord((void **)&replicaRecord, "ReplicaRecord",
-                sizeof(ReplicaRecord), 
-                creplicaFileSize);
-  
   deallocRecord((void **)&tabRecord, "TabRecord",
                 sizeof(TabRecord), 
                 ctabFileSize);
