@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -62,6 +69,22 @@ DbtcProxy::DbtcProxy(Block_context& ctx) :
   addRecSignal(GSN_ABORT_ALL_REF,&DbtcProxy::execABORT_ALL_REF);
   addRecSignal(GSN_ABORT_ALL_CONF,&DbtcProxy::execABORT_ALL_CONF);
 
+  // Routed signals are distributed across the workers
+  // This requires that there's no ordering constraints between them
+  // GSN_TCKEY_FAILREFCONF_R
+  addRecSignal(GSN_TCKEY_FAILREFCONF_R, &DbtcProxy::forwardToAnyWorker);
+
+  // GSN_CREATE_FK_IMPL_REQ
+  addRecSignal(GSN_CREATE_FK_IMPL_REQ, &DbtcProxy::execCREATE_FK_IMPL_REQ);
+  addRecSignal(GSN_CREATE_FK_IMPL_CONF,&DbtcProxy::execCREATE_FK_IMPL_CONF);
+  addRecSignal(GSN_CREATE_FK_IMPL_REF, &DbtcProxy::execCREATE_FK_IMPL_REF);
+
+  // GSN_DROP_FK_IMPL_REQ
+  addRecSignal(GSN_DROP_FK_IMPL_REQ, &DbtcProxy::execDROP_FK_IMPL_REQ);
+  addRecSignal(GSN_DROP_FK_IMPL_CONF,&DbtcProxy::execDROP_FK_IMPL_CONF);
+  addRecSignal(GSN_DROP_FK_IMPL_REF, &DbtcProxy::execDROP_FK_IMPL_REF);
+
+
   m_tc_seize_req_instance = 0;
 }
 
@@ -170,9 +193,9 @@ DbtcProxy::sendTCGETOPSIZECONF(Signal* signal, Uint32 ssId)
 
   signal->theData[0] = ss.m_req[0];
   signal->theData[1] = ss.m_sum;
-  sendSignal(ss.m_req[1], GSN_TCGETOPSIZECONF,
-             signal, 2, JBB);
+  signal->theData[2] = ss.m_req[1];
 
+  sendSignal(DBDIH_REF, GSN_CHECK_LCP_IDLE_ORD, signal, 3, JBB);
   ssRelease<Ss_TCGETOPSIZEREQ>(ssId);
 }
 
@@ -593,4 +616,164 @@ DbtcProxy::sendABORT_ALL_CONF(Signal* signal, Uint32 ssId)
   ssRelease<Ss_ABORT_ALL_REQ>(ssId);
 }
 
+// GSN_CREATE_FK_IMPL_REQ
+
+void
+DbtcProxy::execCREATE_FK_IMPL_REQ(Signal* signal)
+{
+  jamEntry();
+  if (!assembleFragments(signal))
+  {
+    jam();
+    return;
+  }
+
+  const CreateFKImplReq* req = (const CreateFKImplReq*)signal->getDataPtr();
+  Ss_CREATE_FK_IMPL_REQ& ss = ssSeize<Ss_CREATE_FK_IMPL_REQ>();
+  ss.m_req = *req;
+  SectionHandle handle(this, signal);
+  saveSections(ss, handle);
+  sendREQ(signal, ss);
+}
+
+void
+DbtcProxy::sendCREATE_FK_IMPL_REQ(Signal* signal, Uint32 ssId,
+                                  SectionHandle * handle)
+{
+  Ss_CREATE_FK_IMPL_REQ& ss = ssFind<Ss_CREATE_FK_IMPL_REQ>(ssId);
+
+  CreateFKImplReq* req = (CreateFKImplReq*)signal->getDataPtrSend();
+  *req = ss.m_req;
+  req->senderRef = reference();
+  req->senderData = ssId;
+  sendSignalNoRelease(workerRef(ss.m_worker), GSN_CREATE_FK_IMPL_REQ,
+                      signal, CreateFKImplReq::SignalLength, JBB,
+                      handle);
+}
+
+void
+DbtcProxy::execCREATE_FK_IMPL_CONF(Signal* signal)
+{
+  const CreateFKImplConf* conf = (const CreateFKImplConf*)signal->getDataPtr();
+  Uint32 ssId = conf->senderData;
+  Ss_CREATE_FK_IMPL_REQ& ss = ssFind<Ss_CREATE_FK_IMPL_REQ>(ssId);
+  recvCONF(signal, ss);
+}
+
+void
+DbtcProxy::execCREATE_FK_IMPL_REF(Signal* signal)
+{
+  const CreateFKImplRef* ref = (const CreateFKImplRef*)signal->getDataPtr();
+  Uint32 ssId = ref->senderData;
+  Ss_CREATE_FK_IMPL_REQ& ss = ssFind<Ss_CREATE_FK_IMPL_REQ>(ssId);
+  recvREF(signal, ss, ref->errorCode);
+}
+
+void
+DbtcProxy::sendCREATE_FK_IMPL_CONF(Signal* signal, Uint32 ssId)
+{
+  Ss_CREATE_FK_IMPL_REQ& ss = ssFind<Ss_CREATE_FK_IMPL_REQ>(ssId);
+  BlockReference dictRef = ss.m_req.senderRef;
+
+  if (!lastReply(ss))
+    return;
+
+  if (ss.m_error == 0)
+  {
+    jam();
+    CreateFKImplConf* conf = (CreateFKImplConf*)signal->getDataPtrSend();
+    conf->senderRef = reference();
+    conf->senderData = ss.m_req.senderData;
+    sendSignal(dictRef, GSN_CREATE_FK_IMPL_CONF,
+               signal, CreateFKImplConf::SignalLength, JBB);
+  }
+  else
+  {
+    jam();
+    CreateFKImplRef* ref = (CreateFKImplRef*)signal->getDataPtrSend();
+    ref->senderRef = reference();
+    ref->senderData = ss.m_req.senderData;
+    ref->errorCode = ss.m_error;
+    sendSignal(dictRef, GSN_CREATE_FK_IMPL_REF,
+               signal, CreateFKImplRef::SignalLength, JBB);
+  }
+
+  ssRelease<Ss_CREATE_FK_IMPL_REQ>(ssId);
+}
+
+// GSN_DROP_FK_IMPL_REQ
+
+void
+DbtcProxy::execDROP_FK_IMPL_REQ(Signal* signal)
+{
+  const DropFKImplReq* req = (const DropFKImplReq*)signal->getDataPtr();
+  Ss_DROP_FK_IMPL_REQ& ss = ssSeize<Ss_DROP_FK_IMPL_REQ>();
+  ss.m_req = *req;
+  ndbrequire(signal->getLength() == DropFKImplReq::SignalLength);
+  sendREQ(signal, ss);
+}
+
+void
+DbtcProxy::sendDROP_FK_IMPL_REQ(Signal* signal, Uint32 ssId, SectionHandle*)
+{
+  Ss_DROP_FK_IMPL_REQ& ss = ssFind<Ss_DROP_FK_IMPL_REQ>(ssId);
+
+  DropFKImplReq* req = (DropFKImplReq*)signal->getDataPtrSend();
+  *req = ss.m_req;
+  req->senderRef = reference();
+  req->senderData = ssId;
+  sendSignal(workerRef(ss.m_worker), GSN_DROP_FK_IMPL_REQ,
+             signal, DropFKImplReq::SignalLength, JBB);
+}
+
+void
+DbtcProxy::execDROP_FK_IMPL_CONF(Signal* signal)
+{
+  const DropFKImplConf* conf = (const DropFKImplConf*)signal->getDataPtr();
+  Uint32 ssId = conf->senderData;
+  Ss_DROP_FK_IMPL_REQ& ss = ssFind<Ss_DROP_FK_IMPL_REQ>(ssId);
+  recvCONF(signal, ss);
+}
+
+void
+DbtcProxy::execDROP_FK_IMPL_REF(Signal* signal)
+{
+  const DropFKImplRef* ref = (const DropFKImplRef*)signal->getDataPtr();
+  Uint32 ssId = ref->senderData;
+  Ss_DROP_FK_IMPL_REQ& ss = ssFind<Ss_DROP_FK_IMPL_REQ>(ssId);
+  recvREF(signal, ss, ref->errorCode);
+}
+
+void
+DbtcProxy::sendDROP_FK_IMPL_CONF(Signal* signal, Uint32 ssId)
+{
+  Ss_DROP_FK_IMPL_REQ& ss = ssFind<Ss_DROP_FK_IMPL_REQ>(ssId);
+  BlockReference dictRef = ss.m_req.senderRef;
+
+  if (!lastReply(ss))
+    return;
+
+  if (ss.m_error == 0)
+  {
+    jam();
+    DropFKImplConf* conf = (DropFKImplConf*)signal->getDataPtrSend();
+    conf->senderRef = reference();
+    conf->senderData = ss.m_req.senderData;
+    sendSignal(dictRef, GSN_DROP_FK_IMPL_CONF,
+               signal, DropFKImplConf::SignalLength, JBB);
+  }
+  else
+  {
+    jam();
+    DropFKImplRef* ref = (DropFKImplRef*)signal->getDataPtrSend();
+    ref->senderRef = reference();
+    ref->senderData = ss.m_req.senderData;
+    ref->errorCode = ss.m_error;
+    sendSignal(dictRef, GSN_DROP_FK_IMPL_REF,
+               signal, DropFKImplRef::SignalLength, JBB);
+  }
+
+  ssRelease<Ss_DROP_FK_IMPL_REQ>(ssId);
+}
+
 BLOCK_FUNCTIONS(DbtcProxy)
