@@ -101,7 +101,6 @@ Dbtup::Dbtup(Block_context& ctx, Uint32 instanceNumber)
   addRecSignal(GSN_SEND_PACKED, &Dbtup::execSEND_PACKED, true);
   addRecSignal(GSN_STTOR, &Dbtup::execSTTOR);
   addRecSignal(GSN_MEMCHECKREQ, &Dbtup::execMEMCHECKREQ);
-  addRecSignal(GSN_TUPKEYREQ, &Dbtup::execTUPKEYREQ);
   addRecSignal(GSN_TUPSEIZEREQ, &Dbtup::execTUPSEIZEREQ);
   addRecSignal(GSN_TUPRELEASEREQ, &Dbtup::execTUPRELEASEREQ);
   addRecSignal(GSN_STORED_PROCREQ, &Dbtup::execSTORED_PROCREQ);
@@ -523,7 +522,7 @@ void Dbtup::execREAD_CONFIG_REQ(Signal* signal)
     c_crashOnCorruptedTuple = val ? true : false;
   }
 
-}//Dbtup::execSIZEALT_REP()
+}
 
 void Dbtup::initRecords() 
 {
@@ -668,7 +667,6 @@ void Dbtup::execNDB_STTOR(Signal* signal)
     break;
   case ZSTARTPHASE3:
     jam();
-    startphase3Lab(signal, ~0, ~0);
     break;
   case ZSTARTPHASE4:
     jam();
@@ -685,10 +683,6 @@ void Dbtup::execNDB_STTOR(Signal* signal)
   sendSignal(cntrRef, GSN_NDB_STTORRY, signal, 1, JBB);
 }//Dbtup::execNDB_STTOR()
 
-void Dbtup::startphase3Lab(Signal* signal, Uint32 config1, Uint32 config2) 
-{
-}//Dbtup::startphase3Lab()
-
 void Dbtup::initializeDefaultValuesFrag()
 {
   /* Grab and initialize a fragment record for storing default
@@ -700,7 +694,8 @@ void Dbtup::initializeDefaultValuesFrag()
   DefaultValuesFragment.p->m_lcp_scan_op = RNIL;
   DefaultValuesFragment.p->noOfPages = 0;
   DefaultValuesFragment.p->noOfVarPages = 0;
-  DefaultValuesFragment.p->m_max_page_no = 0;
+  DefaultValuesFragment.p->m_varWordsFree = 0;
+  DefaultValuesFragment.p->m_max_page_cnt = 0;
   DefaultValuesFragment.p->m_free_page_id_list = FREE_PAGE_RNIL;
   ndbrequire(DefaultValuesFragment.p->m_page_map.isEmpty());
   DefaultValuesFragment.p->m_restore_lcp_id = RNIL;
@@ -872,13 +867,13 @@ void Dbtup::execTUPSEIZEREQ(Signal* signal)
 
   new (regOperPtr.p) Operationrec();
   regOperPtr.p->m_any_value = 0;
-  regOperPtr.p->op_struct.op_type = ZREAD;
-  regOperPtr.p->op_struct.in_active_list = false;
+  regOperPtr.p->op_type = ZREAD;
+  regOperPtr.p->op_struct.bit_field.in_active_list = false;
   set_trans_state(regOperPtr.p, TRANS_DISCONNECTED);
   regOperPtr.p->prevActiveOp = RNIL;
   regOperPtr.p->nextActiveOp = RNIL;
-  regOperPtr.p->tupVersion = ZNIL;
-  regOperPtr.p->op_struct.delete_insert_flag = false;
+  regOperPtr.p->op_struct.bit_field.tupVersion = ZNIL;
+  regOperPtr.p->op_struct.bit_field.delete_insert_flag = false;
   
   initOpConnection(regOperPtr.p);
   regOperPtr.p->userpointer = userPtr;
