@@ -1,19 +1,26 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef SIMBLOCKASYNCFILESYSTEM_H
 #define SIMBLOCKASYNCFILESYSTEM_H
@@ -117,8 +124,23 @@ private:
 
   Uint32 m_bound_threads_cnt;
   Uint32 m_unbounds_threads_cnt;
+
+  /**
+   * Maintaining active bound threads count in NdbFS from *before* a
+   * request is queued (getIdleFile()) until *after* it is
+   * completed (pushIdleFile()) means that idle threads which have
+   * completed requests are not available for further requests until
+   * NdbFS has received and processed their responses.
+
+   * This lag should be fairly short as a wakeup socket is used to
+   * wake NdbFs when a response is available.
+
+   * Any existing code relying on threads more 'quickly' becoming
+   * available for new requests is timing based and therefore likely
+   * to be unreliable.
+   */
   Uint32 m_active_bound_threads_cnt;
-  void cnt_active_bound(int val);
+
 public:
   const BaseString& get_base_path(Uint32 no) const;
 };
