@@ -1,19 +1,26 @@
 /*
-   Copyright (c) 2005, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef PGMAN_H
 #define PGMAN_H
@@ -438,7 +445,8 @@ private:
       m_log_waits(0),
       m_page_requests_direct_return(0),
       m_page_requests_wait_q(0),
-      m_page_requests_wait_io(0)
+      m_page_requests_wait_io(0),
+      m_entries_high(0)
     {}
     Uint32 m_num_pages;         // current number of cache pages
     Uint32 m_num_hot_pages;
@@ -452,6 +460,7 @@ private:
     Uint64 m_page_requests_direct_return;
     Uint64 m_page_requests_wait_q;
     Uint64 m_page_requests_wait_io;
+    Uint32 m_entries_high;
   } m_stats;
 
   enum CallbackIndex {
@@ -469,7 +478,7 @@ protected:
   void execCONTINUEB(Signal* signal);
 
   void execLCP_FRAG_ORD(Signal*);
-  void execEND_LCP_REQ(Signal*);
+  void execEND_LCPREQ(Signal*);
   void execRELEASE_PAGES_REQ(Signal*);
   
   void execFSREADCONF(Signal*);
@@ -502,7 +511,7 @@ private:
   void lirs_reference(Ptr<Page_entry> ptr);
 
   void do_stats_loop(Signal*);
-  void do_busy_loop(Signal*, bool direct = false);
+  void do_busy_loop(Signal*, bool direct, EmulatedJamBuffer *jamBuf);
   void do_cleanup_loop(Signal*);
   void do_lcp_loop(Signal*);
 
