@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -75,7 +75,9 @@ systemInfo(const Configuration & config, const LogLevel & logLevel)
   }
 #elif defined NDB_SOLARIS
   // Search for at max 16 processors among the first 256 processor ids
-  processor_info_t pinfo; memset(&pinfo, 0, sizeof(pinfo));
+  processor_info_t pinfo;
+
+  memset(&pinfo, 0, sizeof(pinfo));
   int pid = 0;
   while(processors < 16 && pid < 256){
     if(!processor_info(pid++, &pinfo))
@@ -187,11 +189,86 @@ compute_acc_32kpages(const ndb_mgm_configuration_iterator * p)
       lqhInstances = globalData.ndbMtLqhWorkers;
     }
     
-    accmem += lqhInstances * (32 / 4); // Added as safty in Configuration.cpp
+    accmem += lqhInstances * (32 / 4); // Added as safety in Configuration.cpp
   }
   return Uint32(accmem);
 }
 
+/**
+ * We currently allocate the following large chunks of memory:
+ * -----------------------------------------------------------
+ *
+ * RG_DATAMEM:
+ * This is a resource which one sets the min == max. This means that
+ * we cannot overallocate this resource. The size of the resource
+ * is based on the sum of the configuration variables DataMemory
+ * and IndexMemory. It's used for main memory tuples, indexes and
+ * hash indexes. We add an extra 8 32kB pages for safety reasons
+ * if IndexMemory is set.
+ *
+ * RG_FILE_BUFFERS:
+ * This is a resource used by the REDO log handler in DBLQH. It is
+ * also a resource we cannot overallocate. The size of it is based
+ * on the multiplication of the config variables NoOfLogFileParts
+ * and RedoBuffer. In addition we add a constant 1 MByte per each
+ * log file part to handle some extra outstanding requests.
+ *
+ * RG_JOB_BUFFERS:
+ * This is a resource used to allocate job buffers by multithreaded
+ * NDB scheduler to handle various job buffers and alike. It has
+ * a complicated algorithm to calculate its size. It allocates a
+ * bit more than 2 MByte per thread and it also allocates a 1 MByte
+ * buffer in both directions for all threads that can communicate.
+ * For large configurations this becomes a fairly large memory
+ * that can consume up to a number of GBytes of memory. It is
+ * also a resource that cannot be overallocated.
+ *
+ * RG_TRANSPORTER_BUFFERS:
+ * This is a resource used for send buffers in ndbmtd. It is set to
+ * a size of the sum of TotalSendBufferMemory and ExtraSendBufferMemory.
+ * It is a resource that can be overallocated by 25%.
+ * TotalSendBufferMemory is by default set to 0. In this the this
+ * variable is calculated by summing the send buffer memory per node.
+ * The default value per send buffer per node is 2 MByte. So this
+ * means that in a system with 4 data nodes and 8 client nodes the
+ * data nodes will have 11 * 2 MByte of total memory. The extra
+ * memory is by default 0. However for ndbmtd we also add more memory
+ * in the extra part. We add 2 MBytes per thread used in the node.
+ * So with 4 LDM threads, 2 TC threads, 1 main thread, 1 rep thread,
+ * and 2 receive threads then we have 10 threads and allocate another
+ * extra 20 MBytes. The user can never set this below 16MByte +
+ * 2 MByte per thread + 256 kByte per node. So default setting is
+ * 2MByte * (#nodes + #threads) and we can oversubscribe by 25% by
+ * using the SharedGlobalMemory.
+ *
+ * RG_DISK_PAGE_BUFFER:
+ * This is a resource that is used for the disk page buffer. It cannot
+ * be overallocated. Its size is calculated based on the config variable
+ * DiskPageBufferMemory.
+ * 
+ * RG_SCHEMA_TRANS_MEMORY:
+ * This is a resource that is set to a minimum of 2 MByte. It can be
+ * overallocated at any size as long as there is still memory
+ * remaining.
+ *
+ * RG_DISK_OPERATIONS:
+ * This is a resource that is either set to zero size but can be overallocated
+ * without limit. If a log file group is allocated based on the config, then
+ * the size of the UNDO log buffer is used to set the size of this resource.
+ * This resource is only used to allocate the UNDO log buffer of an UNDO log
+ * file group and there can only be one such group. It is using overallocating
+ * if this happens through an SQL command.
+ *
+ * Overallocating and total memory
+ * -------------------------------
+ * The total memory allocated by the global memory manager is the sum of the
+ * sizes of the above resources. On top of this one also adds the global
+ * shared memory resource. The size of this is set to the config variable
+ * SharedGlobalMemory. The global shared memory resource is the resource used
+ * when we're overallocating as is currently possible for the UNDO log
+ * memory and also for schema transaction memory. GlobalSharedMemory cannot
+ * be set lower than 128 MByte.
+ */
 static int
 init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
 {
@@ -295,8 +372,14 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
   }
 
   Uint32 sbpages = 0;
-  if (globalTransporterRegistry.get_using_default_send_buffer() == false)
+  if (globalData.isNdbMt)
   {
+    /**
+     * This path is normally always taken for ndbmtd as the transporter
+     * registry defined in mt.cpp is hard coded to set this to false.
+     * For ndbd it is hard coded similarly to be set to true in
+     * TransporterCallback.cpp. So for ndbd this code isn't executed.
+     */
     Uint64 mem;
     {
       Uint32 tot_mem = 0;
@@ -735,6 +818,21 @@ ndbd_run(bool foreground, int report_fd,
     }
   }
 
+  if (initialstart)
+  {
+    g_eventLogger->info("Performing partial initial start of this Cluster");
+  }
+  else if (initial)
+  {
+    g_eventLogger->info(
+      "Initial start of data node, ignoring any info on disk");
+  }
+  else
+  {
+    g_eventLogger->info(
+      "Normal start of data node using checkpoint and log info if existing");
+  }
+
   globalEmulatorData.create();
 
   Configuration* theConfig = globalEmulatorData.theConfiguration;
@@ -744,10 +842,23 @@ ndbd_run(bool foreground, int report_fd,
     ndbd_exit(-1);
   }
 
+  /**
+    Read the configuration from the assigned management server (could be
+    a set of management servers, normally when we arrive here we have
+    already assigned the nodeid, either by the operator or by the angel
+    process.
+  */
   theConfig->fetch_configuration(connect_str, force_nodeid, bind_address,
                                  allocated_nodeid, connect_retries,
                                  connect_delay);
 
+  /**
+    Set the NDB DataDir, this is where we will locate log files and data
+    files unless specifically configured to be elsewhere.
+  */
+  g_eventLogger->info("Changing directory to '%s'",
+                      NdbConfig_get_path(NULL));
+
   if (NdbDir::chdir(NdbConfig_get_path(NULL)) != 0)
   {
     g_eventLogger->warning("Cannot change directory to '%s', error: %d",
@@ -757,13 +868,24 @@ ndbd_run(bool foreground, int report_fd,
 
   theConfig->setupConfiguration();
 
+
+  /**
+    Printout various information about the threads in the
+    run-time environment
+  */
   if (get_multithreaded_config(globalEmulatorData))
     ndbd_exit(-1);
-
   systemInfo(* theConfig, * theConfig->m_logLevel);
 
+  /**
+    Start the watch-dog thread before we start allocating memory.
+    Allocation of memory can be a very time-consuming process.
+    The watch-dog will have a special timeout for the phase where
+    we allocate memory.
+  */
   NdbThread* pWatchdog = globalEmulatorData.theWatchDog->doStart();
 
+  g_eventLogger->info("Memory Allocation for global memory pools Starting");
   {
     /*
      * Memory allocation can take a long time for large memory.
@@ -777,7 +899,14 @@ ndbd_run(bool foreground, int report_fd,
       ndbd_exit(1);
     globalEmulatorData.theWatchDog->unregisterWatchedThread(0);
   }
+  g_eventLogger->info("Memory Allocation for global memory pools Completed");
 
+  /**
+    Initialise the data of the run-time environment, this prepares the
+    data setup for the various threads that need to communicate using
+    our internal memory. The threads haven't started yet, but as soon as
+    they start they will be ready to communicate.
+  */
   globalEmulatorData.theThreadConfig->init();
 
 #ifdef VM_TRACE
@@ -813,18 +942,18 @@ ndbd_run(bool foreground, int report_fd,
   NdbMem_Free(signal_log_name);
 #endif
 
+  /** Create all the blocks used by the run-time environment. */
+  g_eventLogger->info("Loading blocks for data node run-time environment");
   // Load blocks (both main and workers)
   globalEmulatorData.theSimBlockList->load(globalEmulatorData);
 
-  // Set thread concurrency for Solaris' light weight processes
-  int status;
-  status = NdbThread_SetConcurrencyLevel(30);
-  assert(status == 0);
-
   catchsigs(foreground);
 
   /**
-   * Do startup
+   * Send the start signal to the CMVMI block. The start will however
+   * not start until we have started the thread that runs the CMVMI
+   * block. As soon as this thread starts it will find the signal
+   * there to execute and we can start executing signals.
    */
   switch(globalData.theRestartFlag){
   case initial_state:
@@ -838,36 +967,74 @@ ndbd_run(bool foreground, int report_fd,
     assert("Illegal state globalData.theRestartFlag" == 0);
   }
 
+  /**
+    Before starting the run-time environment we also need to activate the
+    send and receive services. We need for some cases to prepare some data
+    in the TransporterRegistry before we start the communication service.
+    The connection to the management server is reused as a connection to
+    the management server node.
+    The final steps is to start the client connections, these are all the
+    nodes that we need to be client in the communication setup. Then start
+    the socket server where other nodes can connect to us for those nodes
+    where our node is the server part of the connection. The logic is that
+    the node with the lower nodeid is the server and the other one the client,
+    this can be changed by the configuration. This is implemented by the
+    management server in the function fixPortNumber.
+  */
+  g_eventLogger->info("Starting Sending and Receiving services");
   globalTransporterRegistry.startSending();
   globalTransporterRegistry.startReceiving();
   if (!globalTransporterRegistry.start_service(*globalEmulatorData.m_socket_server)){
     ndbout_c("globalTransporterRegistry.start_service() failed");
     ndbd_exit(-1);
   }
-
   // Re-use the mgm handle as a transporter
   if(!globalTransporterRegistry.connect_client(
 		 theConfig->get_config_retriever()->get_mgmHandlePtr()))
       ERROR_SET(fatal, NDBD_EXIT_CONNECTION_SETUP_FAILED,
                 "Failed to convert mgm connection to a transporter",
                 __FILE__);
-
   NdbThread* pTrp = globalTransporterRegistry.start_clients();
   if (pTrp == 0)
   {
     ndbout_c("globalTransporterRegistry.start_clients() failed");
     ndbd_exit(-1);
   }
-
   NdbThread* pSockServ = globalEmulatorData.m_socket_server->startServer();
 
+  /**
+    Report the new threads started, there is one thread started now to handle
+    the watchdog, one to handle the socket server part and one to regularly
+    attempt to connect as client to other nodes.
+  */
   globalEmulatorData.theConfiguration->addThread(pTrp, SocketClientThread);
   globalEmulatorData.theConfiguration->addThread(pWatchdog, WatchDogThread);
   globalEmulatorData.theConfiguration->addThread(pSockServ, SocketServerThread);
+
+  g_eventLogger->info("Starting the data node run-time environment");
   {
+    /**
+      We have finally arrived at the point where we start the run-time
+      environment, in this method we will create the needed threads.
+      We still have two different ThreadConfig objects, one to run
+      ndbd (the single threaded variant of the data node process) and
+      one to run ndbmtd (the multithreaded variant of the data node
+      process).
+
+      Mostly the ndbmtd should be used, but there could still be
+      cases where someone prefers the single-threaded variant since
+      this can provide lower latency if throughput isn't an issue.
+    */
     NdbThread *pThis = NdbThread_CreateObject(0);
     globalEmulatorData.theThreadConfig->ipControlLoop(pThis);
   }
+  g_eventLogger->info("The data node run-time environment has been stopped");
+
+  /**
+    The data node process is stopping, we remove the watchdog thread, the
+    socket server and socket client thread from the list of running
+    threads.
+  */
   globalEmulatorData.theConfiguration->removeThread(pWatchdog);
   globalEmulatorData.theConfiguration->removeThread(pTrp);
   globalEmulatorData.theConfiguration->removeThread(pSockServ);
