@@ -60,6 +60,7 @@ public:
   void * getPtr(Uint32 i);
   
 private:  
+  void seize_in_page(Ptr<void>&);
   bool seize_new_page(Ptr<void>&);
   void release_not_current(Ptr<void>);
 
@@ -68,23 +69,34 @@ private:
   void handle_inconsistent_release(Ptr<void>) ATTRIBUTE_NORETURN;
 };
 
+inline
+void
+WOPool::seize_in_page(Ptr<void>& ptr)
+{
+  Uint32 pos = m_current_pos;
+  WOPage *pageP = m_current_page;
+  Uint32 magic_pos = pos + m_record_info.m_offset_magic;
+  Uint32 type_id = ~(Uint32)m_record_info.m_type_id;
+  Uint32 size = m_record_info.m_size;
+  Uint16 ref_count = m_current_ref_count;
+
+  assert(pos + size < WOPage::WOPAGE_WORDS);
+  ptr.i = (m_current_page_no << POOL_RECORD_BITS) + pos;
+  ptr.p = (pageP->m_data + pos);
+  pageP->m_data[magic_pos] = type_id;
+  m_current_pos = pos + size;
+  m_current_ref_count = ref_count + 1;
+}
+
 inline
 bool
 WOPool::seize(Ptr<void>& ptr)
 {
-  Uint32 pos = m_current_pos;
-  Uint32 size = m_record_info.m_size;
-  WOPage *pageP = m_current_page;
-  if (likely(pos + size < WOPage::WOPAGE_WORDS))
+  if (likely(m_current_pos + m_record_info.m_size < WOPage::WOPAGE_WORDS))
   {
-    ptr.i = (m_current_page_no << POOL_RECORD_BITS) + pos;
-    ptr.p = (pageP->m_data + pos);
-    pageP->m_data[pos+m_record_info.m_offset_magic] = ~(Uint32)m_record_info.m_type_id;
-    m_current_pos = pos + size;
-    m_current_ref_count++;
+    seize_in_page(ptr);
     return true;
   }
-  
   return seize_new_page(ptr);
 }
 
