@@ -52,7 +52,8 @@ struct NdbApiConfig
     m_batch_size(DEF_BATCH_SIZE),
     m_waitfor_timeout(120000),
     m_default_queue_option(0),
-    m_default_hashmap_size(0)
+    m_default_hashmap_size(0),
+    m_verbose(0)
     {}
 
   Uint32 m_scan_batch_size;
@@ -61,6 +62,7 @@ struct NdbApiConfig
   Uint32 m_waitfor_timeout; // in milli seconds...
   Uint32 m_default_queue_option;
   Uint32 m_default_hashmap_size;
+  Uint32 m_verbose;
 };
 
 class Ndb_cluster_connection_impl : public Ndb_cluster_connection
@@ -109,6 +111,7 @@ private:
   void connect_thread();
   void set_name(const char *name);
   Uint32 get_db_nodes(Uint8 nodesarray[MAX_NDB_NODES]) const;
+  Uint32 get_unconnected_nodes() const;
 
   int connect(int no_retries,
               int retry_delay_in_seconds,
