@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2004, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -102,6 +109,7 @@ static const char* empty_string = "";
  *   NdbInterpretedCode::interpret_exit_nok(). Do not define internal error 
  *   codes in this range!
  * 20000 - SPJ
+ * 21000 - DICT FK
  */
 
 static
@@ -124,6 +132,8 @@ ErrorBundle ErrorCodes[] = {
   { 839,  DMEC, CV, "Illegal null attribute" },
   { 840,  DMEC, CV, "Trying to set a NOT NULL attribute to NULL" },
   { 893,  HA_ERR_FOUND_DUPP_KEY, CV, "Constraint violation e.g. duplicate value in unique index" },
+  { 255,  HA_ERR_NO_REFERENCED_ROW, CV, "Foreign key constraint violated: No parent row found" },
+  { 256,  HA_ERR_ROW_IS_REFERENCED, CV, "Foreign key constraint violated: Referenced row exists" },
 
   /**
    * Node recovery errors
@@ -175,6 +185,40 @@ ErrorBundle ErrorCodes[] = {
   { 20020, HA_ERR_NO_SUCH_TABLE, SE, "Query table is being dropped" },
   { 20021, HA_ERR_TABLE_DEF_CHANGED, SE, "Query table definition has changed" },
 
+  /**
+   * DICT FK kernel and ndbapi error codes
+   */
+  { 21000, HA_ERR_CANNOT_ADD_FOREIGN, AE, "Create foreign key failed - parent key is primary key and on-update-cascade is not allowed" },
+  /* CreateFKRef + CreateFKImplRef */
+  { 21020, DMEC, TR, "Create foreign key failed in NDB - no more object records" },
+  { 21021, DMEC, IE, "Create foreign key failed in NDB - invalid request" },
+  { 21022, DMEC, SE, "Create foreign key failed in NDB - parent table is not table" },
+  { 21023, DMEC, SE, "Create foreign key failed in NDB - invalid parent table version" },
+  { 21024, DMEC, SE, "Create foreign key failed in NDB - child table is not table" },
+  { 21025, DMEC, SE, "Create foreign key failed in NDB - invalid child table version" },
+  { 21026, HA_ERR_CANNOT_ADD_FOREIGN, AE, "Create foreign key failed in NDB - parent index is not unique index" },
+  { 21027, DMEC, SE, "Create foreign key failed in NDB - invalid parent index version" },
+  { 21028, DMEC, SE, "Create foreign key failed in NDB - child index is not index" },
+  { 21029, DMEC, SE, "Create foreign key failed in NDB - invalid child index version" },
+  { 21030, DMEC, IE, "Create foreign key failed in NDB - object already exists in TC" },
+  { 21031, DMEC, IE, "Create foreign key failed in NDB - no more object records in TC" },
+  { 21032, DMEC, IE, "Create foreign key failed in NDB - invalid request to TC" },
+  { 21034, HA_ERR_CANNOT_ADD_FOREIGN, AE, "Create foreign key failed - child table has Blob or Text column and on-delete-cascade is not allowed" },
+  /* DropFKRef + DropFKImplRef */
+  { 21040, DMEC, AE, "Drop foreign key failed in NDB - foreign key not found" },
+  { 21041, DMEC, SE, "Drop foreign key failed in NDB - invalid foreign key version" },
+  { 21042, DMEC, SE, "Drop foreign key failed in NDB - foreign key not found in TC" },
+  /* BuildFKRef + BuildFKImplRef */
+  { 21060, DMEC, AE, "Build foreign key failed in NDB - foreign key not found" },
+  { 21061, DMEC, SE, "Build foreign key failed in NDB - invalid foreign key version" },
+  /* Referential integrity */
+  { 21080, HA_ERR_ROW_IS_REFERENCED, SE, "Drop table not allowed in NDB - referenced by foreign key on another table" },
+  /* Drop index */
+  { 21081, HA_ERR_DROP_INDEX_FK, AE, "Drop index not allowed in NDB - used as parent index of a foreign key" },
+  { 21082, HA_ERR_DROP_INDEX_FK, AE, "Drop index not allowed in NDB - used as child index of a foreign key" },
+  /* Misc */
+  { 21090, HA_ERR_CANNOT_ADD_FOREIGN, AE, "Create foreign key failed in NDB - name contains invalid character (/)" },
+  
   /**
    * Node shutdown
    */
@@ -320,6 +364,7 @@ ErrorBundle ErrorCodes[] = {
   { 230,  DMEC, IE, "230" },
   { 232,  DMEC, IE, "232" },
   { 238,  DMEC, IE, "238" },
+  { 240,  DMEC, IE, "Invalid data encountered during foreign key trigger execution" },
   { 271,  DMEC, IE, "Simple Read transaction without any attributes to read" },
   { 272,  DMEC, IE, "Update operation without any attributes to update" },
   { 276,  DMEC, IE, "276" },
@@ -402,6 +447,7 @@ ErrorBundle ErrorCodes[] = {
   { 322,  DMEC, AE, "Invalid node(s) specified for new nodegroup, node already in nodegroup" },
   { 323,  DMEC, AE, "Invalid nodegroup id, nodegroup already existing" },
   { 324,  DMEC, AE, "Invalid node(s) specified for new nodegroup, no node in nodegroup is started" },
+  { 325,  DMEC, AE, "Invalid node(s) specified for new nodegroup, node ID invalid or undefined" },
   { 417,  DMEC, AE, "Bad operation reference - double unlock" },
 
   /** 
@@ -530,7 +576,7 @@ ErrorBundle ErrorCodes[] = {
 
   { 1502, DMEC, IE, "Filegroup already exists" },
   { 1503, DMEC, SE, "Out of filegroup records" },
-  { 1504, DMEC, SE, "Out of logbuffer memory" },
+  { 1504, DMEC, SE, "Out of logbuffer memory(specify smaller undo_buffer_size or increase SharedGlobalMemory)" },
   { 1505, DMEC, IE, "Invalid filegroup" },
   { 1506, DMEC, IE, "Invalid filegroup version" },
   { 1507, DMEC, IE, "File no already inuse" },
@@ -600,6 +646,7 @@ ErrorBundle ErrorCodes[] = {
   { 1702, DMEC, AE, "Node already connected" },
   { 1703, DMEC, IT, "Node failure handling not completed" },
   { 1704, DMEC, AE, "Node type mismatch" },
+  { 1705, DMEC, IT, "Not ready for connection allocation yet" },
 
   /*
    * Index stats error codes
@@ -653,6 +700,7 @@ ErrorBundle ErrorCodes[] = {
   { 4104, DMEC, AE, "Ndb Init in wrong state, destroy Ndb object and create a new" },
   { 4121, DMEC, AE, "Cannot set name twice for an Ndb object" },
   { 4122, DMEC, AE, "Cannot set name after Ndb object is initialised" },
+  { 4123, DMEC, AE, "Free percent out of range. Allowed range is 1-99" },
   { 4105, DMEC, AE, "Too many Ndb objects" },
   { 4106, DMEC, AE, "All Not NULL attribute have not been defined" },
   { 4114, DMEC, AE, "Transaction is already completed" },
@@ -689,12 +737,14 @@ ErrorBundle ErrorCodes[] = {
   { 4328, DMEC, AE, "Disk memory attributes not yet supported" },
   { 4329, DMEC, AE, "Variable stored attributes not yet supported" },
   { 4340, DMEC, AE, "Result or attribute record must be a base table ndbrecord, not an index ndbrecord" },
+  { 4342, DMEC, AE, "Scan defined but not prepared" },
 
   { 4400, DMEC, AE, "Status Error in NdbSchemaCon" },
   { 4401, DMEC, AE, "Only one schema operation per schema transaction" },
   { 4402, DMEC, AE, "No schema operation defined before calling execute" },
   { 4410, DMEC, AE, "Schema transaction is already started" },
   { 4411, DMEC, AE, "Schema transaction not possible until upgrade complete" },
+  { 4412, DMEC, AE, "Schema transaction is not started" },
 
   { 4501, DMEC, AE, "Insert in hash table failed when getting table information from Ndb" },
   { 4502, DMEC, AE, "GetValue not allowed in Update operation" },
