@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2012, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,8 +24,12 @@
 #include <NdbTick.h>
 #include <NDBT_Stats.hpp>
 #include <random.h>
+#include <NdbMgmd.hpp>
+#include "../../src/ndbapi/NdbInfo.hpp"
 
-#include "storage/ndb/src/mgmapi/mgmapi_configuration.hpp"
+#ifndef UINT32_MAX
+#define UINT32_MAX (4294967295U)
+#endif 
 
 static NdbMutex* g_msgmutex = 0;
 
@@ -57,6 +61,15 @@ static NdbMutex* g_msgmutex = 0;
     break; \
   }
 
+#define CHK3(b, e) \
+  if (!(b)) { \
+    NdbMutex_Lock(g_msgmutex); \
+    g_err << "ERROR: " << #b << " failed at line " << __LINE__ \
+          << ": " << e << endl; \
+    NdbMutex_Unlock(g_msgmutex); \
+    return NDBT_FAILED;          \
+  }
+
 #define info(x) \
   do { \
     NdbMutex_Lock(g_msgmutex); \
@@ -214,7 +227,7 @@ runLongtrans(NDBT_Context* ctx, NDBT_Step* step)
 }
 
 static int
-run_write_ops(NDBT_Context* ctx, NDBT_Step* step, int upval, NdbError& err)
+run_write_ops(NDBT_Context* ctx, NDBT_Step* step, int upval, NdbError& err, bool abort_on_error=false)
 {
   Ndb* pNdb = GETNDB(step);
   const int records = ctx->getNumRecords();
@@ -245,6 +258,13 @@ run_write_ops(NDBT_Context* ctx, NDBT_Step* step, int upval, NdbError& err)
     require(err.code != 0);
     CHK2(err.status == NdbError::TemporaryError, err);
 
+    if (abort_on_error) 
+    {
+      g_err << "Temporary error " << err.code << " during write" << endl;
+      result = NDBT_FAILED;
+      break;
+    }
+
     if (err.code == 410)
       break;
 
@@ -399,9 +419,7 @@ run_latency_ops(NDBT_Context* ctx, NDBT_Step* step, OpLat& oplat, int upval, Ndb
     HugoOperations ops(*pTab);
     ops.setQuiet();
 
-    MicroSecondTimer timer_start;
-    MicroSecondTimer timer_stop;
-    require(NdbTick_getMicroTimer(&timer_start) == 0);
+    const NDB_TICKS timer_start = NdbTick_getCurrentTicks();
 
     CHK2(ops.startTransaction(pNdb) == 0, ops.getNdbError());
 
@@ -435,8 +453,8 @@ run_latency_ops(NDBT_Context* ctx, NDBT_Step* step, OpLat& oplat, int upval, Ndb
       CHK2(err.status == NdbError::TemporaryError, err);
     }
 
-    require(NdbTick_getMicroTimer(&timer_stop) == 0);
-    NDB_TICKS tt = NdbTick_getMicrosPassed(timer_start, timer_stop);
+    const NDB_TICKS timer_stop = NdbTick_getCurrentTicks();
+    Uint64 tt = NdbTick_Elapsed(timer_start, timer_stop).microSec();
     require(tt > 0);
     double td = (double)tt;
     int i0 = get_err410(ctx);
@@ -1266,6 +1284,472 @@ runResetFD(NDBT_Context* ctx, NDBT_Step* step)
   return result;
 }
 
+static int 
+resizeRedoLog(NDBT_Context* ctx, NDBT_Step* step)
+{
+  int result = NDBT_FAILED;
+  Config conf;
+  NdbRestarter restarter;
+  Uint32 noOfLogFiles = ctx->getProperty("REDOLOGCOUNT", (Uint32)16);
+  Uint32 logFileSize = ctx->getProperty("REDOLOGSIZE", (Uint32)16*1024*1024);
+  Uint32 defaultNoOfLogFiles = 0, defaultLogFileSize = 0;
+  do
+  {
+    NdbMgmd mgmd;
+    if(!mgmd.connect())
+    {
+      g_err << "Failed to connect to ndb_mgmd." << endl;
+      break;
+    }
+    if (!mgmd.get_config(conf))
+    {
+      g_err << "Failed to get config from ndb_mgmd." << endl;
+      break;
+    }
+
+    g_err << "Setting NoOfFragmentLogFiles = " << noOfLogFiles
+          << " and FragmentLogFileSize = " << logFileSize << "..." << endl;
+    ConfigValues::Iterator iter(conf.m_configValues->m_config);
+    for (int nodeid = 1; nodeid < MAX_NODES; nodeid ++)
+    {
+      Uint32 oldValue;
+      if (!iter.openSection(CFG_SECTION_NODE, nodeid))
+        continue;
+      if (iter.get(CFG_DB_NO_REDOLOG_FILES, &oldValue)) 
+      {
+        iter.set(CFG_DB_NO_REDOLOG_FILES, noOfLogFiles);
+        if(defaultNoOfLogFiles == 0)
+        {
+           defaultNoOfLogFiles = oldValue;
+        }
+        else if(oldValue != defaultNoOfLogFiles) 
+        {
+          g_err << "NoOfFragmentLogFiles is not consistent across nodes" << endl;
+          break; 
+        }
+      }
+      if(iter.get(CFG_DB_REDOLOG_FILE_SIZE, &oldValue))
+      {
+        iter.set(CFG_DB_REDOLOG_FILE_SIZE, logFileSize);
+        if(defaultLogFileSize == 0)
+        {
+           defaultLogFileSize = oldValue;
+        }
+        else if(oldValue != defaultLogFileSize) 
+        {
+          g_err << "FragmentLogFileSize is not consistent across nodes" << endl;
+          break; 
+        }
+      }
+      iter.closeSection();
+    }
+ 
+    // Save old values of NoOfFragmentLogFiles and FragmentLogFileSize 
+    ctx->setProperty("REDOLOGCOUNT", (Uint32)defaultNoOfLogFiles);
+    ctx->setProperty("REDOLOGSIZE", (Uint32)defaultLogFileSize);
+  
+    if (!mgmd.set_config(conf))
+    {
+      g_err << "Failed to set config in ndb_mgmd." << endl;
+      break;
+    }
+
+    g_err << "Restarting nodes to apply config change..." << endl;
+    if (restarter.restartAll(true))
+    {
+      g_err << "Failed to restart node." << endl;
+      break;
+    }
+    if (restarter.waitClusterStarted(120) != 0)
+    {
+      g_err << "Failed waiting for node started." << endl;
+      break;
+    }
+    g_err << "Nodes restarted with new config" << endl;
+    result = NDBT_OK;
+  } while (0);
+  return result;
+}
+
+static int
+runWriteWithRedoFull(NDBT_Context* ctx, NDBT_Step* step)
+{
+  int upval = 0;
+  NdbRestarter restarter;
+  Ndb* pNdb = GETNDB(step);
+
+  /**
+   * Ensure we don't use the same record for the open transaction as for the ones
+   * filling up the REDO log. In that case we get into a deadlock, we solve this
+   * by using a different table for the pending transaction.
+   */
+  const NdbDictionary::Table* pTab = g_tabptr[0];
+
+  g_err << "Starting a write and leaving it open so the pending " <<
+           "COMMIT indefinitely delays redo log trimming..." << endl;
+  HugoOperations ops(*pTab);
+  ops.setQuiet();
+  if(ops.startTransaction(pNdb) != 0)
+  { 
+    g_err << "Failed to start transaction: error " << ops.getNdbError() << endl;
+    return NDBT_FAILED;
+  }
+  if(ops.pkWriteRecord(pNdb, 0, 1, upval++) != 0)
+  { 
+    g_err << "Failed to write record: error " << ops.getNdbError() << endl;
+    return NDBT_FAILED;
+  }
+  if(ops.execute_NoCommit(pNdb) != 0)
+  {
+    g_err << "Error: failed to execute NoCommit" << ops.getNdbError() << endl;
+    return NDBT_FAILED;
+  }
+
+  g_err << "Starting PK insert load..." << endl;
+  int loop = 0;
+  int result = NDBT_FAILED;
+  while (!ctx->isTestStopped())
+  {
+    if (loop % 100 == 0)
+    {
+      info("write: loop " << loop);
+    }
+
+    NdbError err;
+    run_write_ops(ctx, step, upval++, err, true);
+    if(err.code == 410)
+    {
+      g_err << "Redo log full, new requests aborted as expected" << endl;
+      result = NDBT_OK;
+      break;
+    }
+    else if(err.code == 266)
+    {
+      g_err << "Error; redo log full, but new requests still allowed to queue" << endl;
+      break;
+    }
+    else if(err.code != 0)
+    {
+      g_err << "Error: write failed with unexpected error " << err.code << endl;
+      break;
+    }
+    loop++;
+  }
+  
+  g_err << "Executing pending COMMIT so that redo log can be trimmed..." << endl;
+  int ret = ops.execute_Commit(pNdb);
+  if(ret != 0)
+  {
+    g_err << "Error: failed to execute commit" << ops.getNdbError() << endl;
+    result = NDBT_FAILED;
+  }
+  ops.closeTransaction(pNdb);
+  return result;
+}
+
+static int
+start_open_transaction(NDBT_Context* ctx, NDBT_Step* step, HugoOperations **ops)
+{
+  /**
+   * Ensure we don't use the same record for the open transaction as for
+   * the ones filling up the REDO log. In that case we get into a deadlock,
+   * we solve this by using a different table for the pending transaction.
+   */
+  const NdbDictionary::Table* pTab = g_tabptr[0];
+
+  g_info << "Starting a write and leaving it open so the pending "
+        << "COMMIT indefinitely delays redo log trimming"
+        << pTab <<endl;
+
+  *ops = new HugoOperations(*pTab);
+  CHK3((*ops) != NULL, "Could not create new HugoOperations");
+
+  (*ops)->setQuiet();
+
+  Ndb* pNdb = GETNDB(step);
+  CHK3((*ops)->startTransaction(pNdb) == 0,
+       "Failed to start transaction: error ");
+  int upval = 0;
+  CHK3((*ops)->pkWriteRecord(pNdb, 0, 1, upval++) == 0,
+       (*ops)->getNdbError());
+  CHK3((*ops)->execute_NoCommit(pNdb) == 0,
+       "Error: failed to execute NoCommit");
+
+  return NDBT_OK;
+}
+
+/**
+ * If the given logpart_with_maxusage and nodeid are invalid,
+ *   return the maximum REDO log usage and the node id and logpart
+ *      which is having it. Fails if two distinct logparts
+ *      (except primary and backup) have same usage.
+ * else return the REDO log usage of the given nodeid and logpart.
+ */
+static int
+get_redo_logpart_maxusage(NDBT_Context* ctx, Uint32 &nodeid,
+                          Uint32 &logpart_with_maxusage)
+{
+  NdbInfo ndbinfo(&ctx->m_cluster_connection, "ndbinfo/");
+  if (!ndbinfo.init())
+  {
+    g_err << "ndbinfo.init failed" << endl;
+    return -1;
+  }
+
+  const NdbInfo::Table* table;
+  if (ndbinfo.openTable("ndbinfo/logspaces", &table) != 0)
+  {
+    g_err << "Failed to openTable(logspaces)" << endl;
+    return -1;
+  }
+
+  NdbInfoScanOperation* scanOp = NULL;
+  if (ndbinfo.createScanOperation(table, &scanOp))
+  {
+    g_err << "No NdbInfoScanOperation" << endl;
+    return -1;
+  }
+
+  if (scanOp->readTuples() != 0)
+  {
+    g_err << "scanOp->readTuples failed" << endl;
+    return -1;
+  }
+
+  const NdbInfoRecAttr* nodeid_colval = scanOp->getValue("node_id");
+  const NdbInfoRecAttr* logtype_colval = scanOp->getValue("log_type");
+  const NdbInfoRecAttr* logpart_colval = scanOp->getValue("log_part");
+  const NdbInfoRecAttr* total_colval = scanOp->getValue("total");
+  const NdbInfoRecAttr* used_colval = scanOp->getValue("used");
+
+  if(scanOp->execute() != 0)
+  {
+    g_err << "scanOp->execute failed" << endl;
+    return -1;
+  }
+
+  // Help variables to trace the max usage and the log part/node id having it
+  int max_usage = -1, usage = -1;
+  Uint32 max_logpart = UINT32_MAX;
+  Uint32 max_node_id = 0;
+
+  while(scanOp->nextResult() == 1)
+  {
+    Uint32 node_id = nodeid_colval->u_32_value();
+    Uint64 total = total_colval->u_64_value();
+    Uint64 used = used_colval->u_64_value();
+    Uint32 logtype = logtype_colval->u_32_value();
+    Uint32 logpart = logpart_colval->u_32_value();
+
+    /* The result row can be skipped if
+     * - it is NOT a redo log data or
+     * - it is NOT the row the test has requested to retrieve
+     */
+    if (logtype != 0 || // Not a redo log
+        (nodeid != 0 && logpart_with_maxusage != UINT32_MAX &&
+         nodeid != node_id && logpart_with_maxusage != logpart))
+    {
+      continue;
+    }
+
+    if (total != 0)
+    {
+      usage = (int)(100 * used / total);
+
+      g_info << "nodeid " << node_id << " " << nodeid
+            << " logpart " << logpart << " " << logpart_with_maxusage
+            << " usage " << usage << " " << max_usage << endl;
+
+      // Requested row is found
+      if (node_id == nodeid && logpart == logpart_with_maxusage)
+      {
+        g_info << "Row with requested nodeid " << nodeid << " and logpart " << logpart
+              << "  is found. Usage " << usage << endl;
+        return usage;
+      }
+
+      /* The test blocks one logpart from being trimmed.
+       * The following check may become true when LCP races with the load.
+       * The probability is less for runCheckLCPStartsAfterSR
+       * than for runCheckLCPStartsAfterNR,
+       * since the latter calls this method without LCPs performed.
+       */
+      if (usage > 0 && usage == max_usage &&
+          max_logpart != logpart && max_node_id != node_id)
+      {
+        g_err << "Two non-peer log parts having same usage is not handled" << endl;
+        return -1;
+      }
+
+      // Find the max usage and the corresponding nodeid/logpart.
+      // Primary and backup logparts will be full. Return the
+      // usage of the last row retrieved from ndbinfo/logspace.
+      if (usage > max_usage)
+      {
+        max_usage = usage;
+        max_logpart = logpart;
+        max_node_id = node_id;
+      }
+    }
+  }
+  ndbinfo.releaseScanOperation(scanOp);
+  ndbinfo.closeTable(table);
+
+  // Return the results
+  logpart_with_maxusage = max_logpart;
+  nodeid = max_node_id;
+
+  g_info << "get_redo_logpart_maxusage returns: nodeid " << nodeid
+        << " lp " << logpart_with_maxusage
+        << " usage " << max_usage << endl;
+
+  if (max_usage <= 0)
+    g_err << " The test could not fill the redo log. Redo log usage : usage " << usage
+          << " max usage " << max_usage << endl;
+
+  return max_usage;
+}
+
+static int
+redologpart_is_trimmed(NDBT_Context* ctx, int usage_before,
+                   Uint32 full_logpart, Uint32 nodeid)
+{
+  // Check whether the redo log is trimmed after system or node restart.
+  // Wait max 2/3 of max LCP_INTERVAL (20) seconds for an lcp to
+  // trim the logpart that was full. Slow machines may need more time.
+  int retries = 20;
+  int usage_after = -1;
+  do
+  {
+    NdbSleep_MilliSleep(1000);
+    usage_after = get_redo_logpart_maxusage(ctx, nodeid, full_logpart);
+    CHK3(usage_after != -1, "Could not retrieve redo log usage");
+    g_info << "Retrying : Usage before : " << usage_before
+          << " Usage after : " << usage_after
+          << " Retries " << 20-retries << endl;
+  } //while (retries-- > 0 && usage_after >= usage_before);
+  while (retries-- > 0 && usage_after > 0);
+
+
+  if (usage_after > 0)
+  {
+    g_err << "Redo log is not trimmed " <<  20 - retries
+          << " seconds after restart. "
+          << " Usage before : " << usage_before
+          << " Usage after : " << usage_after
+          << " logpart " << full_logpart
+          << " nodeid " << nodeid << endl;
+    return NDBT_FAILED;
+  }
+  return NDBT_OK;
+}
+
+/** Test if a delay in opening a redo file is handled gracefully.
+ * Fill the redo log. Delay opening a redo log file in order to
+ * simulate a tardy disk, wth of error insertion (5090).
+ * Empty the redo log, execute some more transactions.
+ *
+ * The test fills some redo log part upto almost full (error 410).
+ * The error insertion is on the first redo log with fileNo>3 and the default
+ * NoOfFragmentLogFiles=16. So the test assumes with confidence that
+ * the error insertion must have occurred and the victim redo log file
+ * is opend after the delay, before the redo log part becomes full.
+ *
+ * The test will fail if transactions get errors other than the following:
+ * 410 - REDO log files overloaded, 266 - Time-out in NDB,
+ * 1220 - REDO log files overloaded (increase FragmentLogFileSize).
+ * The latter will be the direct consequence of the error injection
+ * when the test is run with DefaultOperationRedoProblemAction=abort.
+ * However the test is run with default value "queue".
+ * Test will pass after the redo log is trimmed and some more transactions pass.
+ */
+static int
+runCheckOpenNextRedoLogFile(NDBT_Context* ctx, NDBT_Step* step)
+{
+  // Block redo logpart being trimmed by holding a transaction open
+  HugoOperations *ops = NULL;
+  start_open_transaction(ctx, step, &ops);
+
+  NdbRestarter restarter;
+  int node = restarter.getNode(NdbRestarter::NS_RANDOM);
+  g_err << "Inserting error in node " << node << endl;
+  CHK3(restarter.insertErrorInNode(node, 5090) == 0, "Error insertion failed");
+
+  // Run transactions until some redo log part gets full.
+  // Commit the open transaction to trim the redo log.
+  int retries = -1;
+  int success_after_err = 0;
+  NdbError err;
+  int upval = 0;
+  g_err << "Filling redo logs" << endl;
+  while (!ctx->isTestStopped())
+  {
+    run_write_ops(ctx, step, upval++, err, true);
+
+    if ( err.code == 410)
+    {
+      if (retries == -1)
+        retries = 100;
+
+      // Find the logpart that became almost full
+      int usage_before = -1;
+      Uint32 full_logpart = UINT32_MAX;
+      Uint32 nodeid = 0;
+
+      usage_before = get_redo_logpart_maxusage(ctx, nodeid, full_logpart);
+      CHK3(usage_before> 0, "Redo log usage <= 0");
+      CHK3(nodeid != 0, "No nodeid found with almost full logpart");
+      CHK3(full_logpart != UINT32_MAX, "No logpart became full");
+
+      // Commit the open transaction to trim the redo log part.
+      CHK3(ops->execute_Commit(GETNDB(step)) == 0,
+           "Error: failed to commit the open transaction.");
+
+      g_err << "Check whether the redo log is trimmed" << endl;
+      CHK3(((redologpart_is_trimmed(ctx, usage_before, full_logpart,
+                                    nodeid)) == NDBT_OK),
+           "Check for redolog trimmed failed");
+
+      // Start counting the succeeded transactions after the log part trim
+      success_after_err = 0;
+
+    }
+    else if (err.code == 266 || err.code == 1220)
+    {
+      NdbSleep_MilliSleep(100);
+      // Continue with new transactions
+    }
+    else if (err.code > 0)
+    {
+      g_err << "Transaction aborted with err " << err.code
+            << " " << err.message << endl;
+      break;
+    }
+    else
+    {
+      // err.code = 0 (no errors)
+      if (retries > 0)
+      {
+        if (success_after_err++ > 50)
+        {
+          // Some more transactions are executed to confirm that
+          // the inserted error scenario is alleviated.
+          return NDBT_OK;
+        }
+        if (retries-- == 1)
+        {
+          g_err << "Transactions completed after redo log is trimmed are : "
+                << success_after_err
+                << ", Intended to complete > 50"
+                << endl;
+          break;
+        }
+      }
+    }
+  }
+  return NDBT_FAILED;
+}
+
 NDBT_TESTSUITE(testRedo);
 TESTCASE("WriteOK", 
 	 "Run only write to verify REDO size is adequate"){
@@ -1315,8 +1799,29 @@ TESTCASE("RestartFDSR",
   FINALIZER(runDrop);
   FINALIZER(runResetFD);
 }
-NDBT_TESTSUITE_END(testRedo);
+TESTCASE("RedoFull",
+         "Fill redo logs, apply load and check queuing aborted"){
+  TC_PROPERTY("TABMASK", (Uint32)(3));
+  TC_PROPERTY("REDOLOGCOUNT", (Uint32)(3));
+  TC_PROPERTY("REDOLOGSIZE", (Uint32)(4*1024*1024));
+  INITIALIZER(resizeRedoLog);
+  INITIALIZER(runCreate);
+  STEP(runWriteWithRedoFull);
+  FINALIZER(runDrop);
+  FINALIZER(resizeRedoLog);
+}
+TESTCASE("CheckNextRedoFileOpened",
+         "Fill redo logs to full, check if next file is open"
+         "in a stressed disk situation"){
+  TC_PROPERTY("TABMASK", (Uint32)(3));
+  INITIALIZER(resizeRedoLog);
+  INITIALIZER(runCreate);
+  STEP(runCheckOpenNextRedoLogFile);
+  FINALIZER(runDrop);
+  FINALIZER(resizeRedoLog);
+}
 
+NDBT_TESTSUITE_END(testRedo);
 
 int
 main(int argc, const char** argv)
@@ -1324,7 +1829,7 @@ main(int argc, const char** argv)
   ndb_init();
   NDBT_TESTSUITE_INSTANCE(testRedo);
   testRedo.setCreateTable(false);
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   g_msgmutex = NdbMutex_Create();
   require(g_msgmutex != 0);
   int ret = testRedo.execute(argc, argv);
