@@ -1,31 +1,39 @@
 /*
-   Copyright (c) 2004, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include <time.h>
 #include <ndb_global.h>
 #include <ndb_opts.h>
+#include <time.h>
 
 #include <mgmapi.h>
 #include <NdbMain.h>
 #include <NdbOut.hpp>
 #include <NdbSleep.h>
 #include <NdbTick.h>
+#include <portlib/ndb_localtime.h>
 
-#include <NDBT.hpp>
+#include <NdbToolsProgramExitCodes.hpp>
 
 #include <kernel/NodeBitmask.hpp>
 
@@ -35,7 +43,7 @@ waitClusterStatus(const char* _addr, ndb_mgm_node_status _status);
 static int _no_contact = 0;
 static int _not_started = 0;
 static int _single_user = 0;
-static int _timeout = 120;
+static int _timeout = 120; // Seconds
 static const char* _wait_nodes = 0;
 static const char* _nowait_nodes = 0;
 static NdbNodeBitmask nowait_nodes_bitmask;
@@ -88,7 +96,7 @@ void catch_signal(int signum)
 int main(int argc, char** argv){
   NDB_INIT(argv[0]);
   ndb_opt_set_usage_funcs(short_usage_sub, usage);
-  load_defaults("my",load_default_groups,&argc,&argv);
+  ndb_load_defaults(NULL,load_default_groups,&argc,&argv);
 
 #ifndef DBUG_OFF
   opt_debug= "d:t:O,/tmp/ndb_waiter.trace";
@@ -102,7 +110,7 @@ int main(int argc, char** argv){
 
   if (handle_options(&argc, &argv, my_long_options,
                      ndb_std_get_one_option))
-    return NDBT_ProgramExit(NDBT_WRONGARGS);
+    return NdbToolsProgramExitCode::WRONG_ARGS;
 
   const char* connect_string = argv[0];
   if (connect_string == 0)
@@ -170,8 +178,9 @@ int main(int argc, char** argv){
   }
 
   if (waitClusterStatus(connect_string, wait_status) != 0)
-    return NDBT_ProgramExit(NDBT_FAILED);
-  return NDBT_ProgramExit(NDBT_OK);
+    return NdbToolsProgramExitCode::FAILED;
+
+  return NdbToolsProgramExitCode::OK;
 }
 
 #define MGMERR(h) \
@@ -198,16 +207,16 @@ getStatus(){
       MGMERR(handle);
       retries++;
       ndb_mgm_disconnect(handle);
-      if (ndb_mgm_connect(handle,0,0,1)) {
+      if (ndb_mgm_connect(handle, opt_connect_retries, opt_connect_retry_delay, 1)) {
         MGMERR(handle);
-        g_err  << "Reconnect failed" << endl;
+        ndberr << "Reconnect failed" << endl;
         break;
       }
       continue;
     }
     int count = status->no_of_nodes;
     for (int i = 0; i < count; i++){
-      node = &status->node_states[i];      
+      node = &status->node_states[i];
       switch(node->node_type){
       case NDB_MGM_NODE_TYPE_NDB:
         if (!nowait_nodes_bitmask.get(node->node_id))
@@ -246,25 +255,24 @@ getStatus(){
   return -1;
 }
 
+static
 char*
-getTimeAsString(char* pStr)
+getTimeAsString(char* pStr, size_t len)
 {
+  // Get current time
   time_t now;
-  now= ::time((time_t*)NULL);
+  time(&now);
 
-  struct tm* tm_now;
-#ifdef NDB_WIN32
-  tm_now = localtime(&now);
-#else
-  tm_now = ::localtime(&now); //uses the "current" timezone
-#endif
-
-  BaseString::snprintf(pStr, 9,
-	   "%02d:%02d:%02d",
-	   tm_now->tm_hour,
-	   tm_now->tm_min,
-	   tm_now->tm_sec);
+  // Convert to local timezone
+  tm tm_buf;
+  ndb_localtime_r(&now, &tm_buf);
 
+  // Print to string buffer
+  BaseString::snprintf(pStr, len,
+                       "%02d:%02d:%02d",
+                       tm_buf.tm_hour,
+                       tm_buf.tm_min,
+                       tm_buf.tm_sec);
   return pStr;
 }
 
@@ -281,19 +289,19 @@ waitClusterStatus(const char* _addr,
 
   handle = ndb_mgm_create_handle();
   if (handle == NULL){
-    g_err << "Could not create ndb_mgm handle" << endl;
+    ndberr << "Could not create ndb_mgm handle" << endl;
     return -1;
   }
-  g_info << "Connecting to mgmsrv at " << _addr << endl;
+  ndbout << "Connecting to mgmsrv at " << _addr << endl;
   if (ndb_mgm_set_connectstring(handle, _addr))
   {
     MGMERR(handle);
-    g_err  << "Connectstring " << _addr << " invalid" << endl;
+    ndberr << "Connectstring " << _addr << " invalid" << endl;
     return -1;
   }
-  if (ndb_mgm_connect(handle,0,0,1)) {
+  if (ndb_mgm_connect(handle, opt_connect_retries, opt_connect_retry_delay, 1)) {
     MGMERR(handle);
-    g_err  << "Connection to " << _addr << " failed" << endl;
+    ndberr << "Connection to " << _addr << " failed" << endl;
     return -1;
   }
 
@@ -302,11 +310,12 @@ waitClusterStatus(const char* _addr,
   const int MAX_RESET_ATTEMPTS = 10;
   bool allInState = false;
 
-  Uint64 time_now = NdbTick_CurrentMillisecond();
-  Uint64 timeout_time = time_now + 1000 * _timeout;
+  NDB_TICKS start = NdbTick_getCurrentTicks();
+  NDB_TICKS now = start;
 
   while (allInState == false){
-    if (_timeout > 0 && time_now > timeout_time){
+    if (_timeout > 0 &&
+        NdbTick_Elapsed(start,now).seconds() > (Uint64)_timeout){
       /**
        * Timeout has expired waiting for the nodes to enter
        * the state we want
@@ -316,35 +325,39 @@ waitClusterStatus(const char* _addr,
        * Make special check if we are waiting for
        * cluster to become started
        */
-      if(_status == NDB_MGM_NODE_STATUS_STARTED){
-	waitMore = true;
-	/**
-	 * First check if any node is not starting
-	 * then it's no idea to wait anymore
-	 */
-	for (unsigned n = 0; n < ndbNodes.size(); n++){
-	  if (ndbNodes[n].node_status != NDB_MGM_NODE_STATUS_STARTED &&
-	      ndbNodes[n].node_status != NDB_MGM_NODE_STATUS_STARTING)
-	    waitMore = false;
-
-	}
+      if(_status == NDB_MGM_NODE_STATUS_STARTED)
+      {
+        waitMore = true;
+        /**
+         * First check if any node is not starting
+         * then it's no idea to wait anymore
+         */
+        for (unsigned n = 0; n < ndbNodes.size(); n++)
+        {
+          if (ndbNodes[n].node_status != NDB_MGM_NODE_STATUS_STARTED &&
+              ndbNodes[n].node_status != NDB_MGM_NODE_STATUS_STARTING)
+          {
+            waitMore = false;
+            break;
+          }
+        }
       }
 
       if (!waitMore || resetAttempts > MAX_RESET_ATTEMPTS){
-	g_err << "waitNodeState("
+	ndberr << "waitNodeState("
 	      << ndb_mgm_get_node_status_string(_status)
 	      <<", "<<_startphase<<")"
 	      << " timeout after " << attempts << " attempts" << endl;
 	return -1;
       }
 
-      g_err << "waitNodeState("
+      ndberr << "waitNodeState("
 	    << ndb_mgm_get_node_status_string(_status)
 	    <<", "<<_startphase<<")"
 	    << " resetting timeout "
 	    << resetAttempts << endl;
 
-      timeout_time = time_now + 1000 * _timeout;
+      start = now;
 
       resetAttempts++;
     }
@@ -362,9 +375,9 @@ waitClusterStatus(const char* _addr,
     for (unsigned n = 0; n < ndbNodes.size(); n++) {
       ndb_mgm_node_state* ndbNode = &ndbNodes[n];
 
-      assert(ndbNode != NULL);
+      require(ndbNode != NULL);
 
-      g_info << "Node " << ndbNode->node_id << ": "
+      ndbout << "Node " << ndbNode->node_id << ": "
 	     << ndb_mgm_get_node_status_string(ndbNode->node_status)<< endl;
 
       if (ndbNode->node_status !=  _status)
@@ -372,15 +385,15 @@ waitClusterStatus(const char* _addr,
     }
 
     if (!allInState) {
-      char time[9];
-      g_info << "[" << getTimeAsString(time) << "] "
+      char timestamp[9];
+      ndbout << "[" << getTimeAsString(timestamp, sizeof(timestamp)) << "] "
              << "Waiting for cluster enter state "
              << ndb_mgm_get_node_status_string(_status) << endl;
     }
 
     attempts++;
-    
-    time_now = NdbTick_CurrentMillisecond();
+
+    now = NdbTick_getCurrentTicks();
   }
   return 0;
 }
